/* valasemanticanalyzer.vala
 *
 * Copyright (C) 2006-2007  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <rasa@gmx.ch>
 */
#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))
#define VALA_FREE_UNCHECKED(o,f) ((o) = (f (o), NULL))

#include "valasemanticanalyzer.h"
#include <vala/valasemanticanalyzer.h>
#include <vala/valareport.h>
#include <vala/valasourcereference.h>
#include <vala/valamemberaccessibility.h>
#include <vala/valaattribute.h>
#include <vala/valastatement.h>
#include <vala/valacallback.h>
#include <vala/valaarray.h>
#include <vala/valalockable.h>
#include <vala/valatypeparameter.h>
#include <vala/valaliteral.h>
#include <vala/valaenumvalue.h>
#include <vala/valanamespacereference.h>
#include <vala/valapointer.h>
#include <vala/valaenum.h>
#include <vala/valainvokable.h>
#include <vala/valasignal.h>
#include <string.h>

struct _ValaSemanticAnalyzerPrivate {
	gboolean _memory_management;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaSourceFile* current_source_file;
	ValaTypeReference* current_return_type;
	ValaClass* current_class;
	ValaStruct* current_struct;
	GList* current_using_directives;
	ValaTypeReference* bool_type;
	ValaTypeReference* string_type;
	ValaTypeReference* int_type;
	ValaTypeReference* type_type;
	ValaDataType* pointer_type;
	ValaDataType* initially_unowned_type;
	gint next_lambda_id;
};
#define VALA_SEMANTIC_ANALYZER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzerPrivate))
enum  {
	VALA_SEMANTIC_ANALYZER_DUMMY_PROPERTY,
	VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT,
};
static void vala_semantic_analyzer_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vala_semantic_analyzer_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vala_semantic_analyzer_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_semantic_analyzer_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_semantic_analyzer_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl);
static GList* vala_semantic_analyzer_get_all_prerequisites (ValaSemanticAnalyzer* self, ValaInterface* iface);
static gboolean vala_semantic_analyzer_class_is_a (ValaSemanticAnalyzer* self, ValaClass* cl, ValaDataType* t);
static void vala_semantic_analyzer_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_semantic_analyzer_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_semantic_analyzer_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_semantic_analyzer_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_semantic_analyzer_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_semantic_analyzer_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
static void vala_semantic_analyzer_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vala_semantic_analyzer_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_semantic_analyzer_find_base_class_method (ValaSemanticAnalyzer* self, ValaMethod* m, ValaClass* cl);
static void vala_semantic_analyzer_find_base_interface_method (ValaSemanticAnalyzer* self, ValaMethod* m, ValaClass* cl);
static void vala_semantic_analyzer_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_semantic_analyzer_real_visit_begin_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m);
static void vala_semantic_analyzer_real_visit_end_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m);
static void vala_semantic_analyzer_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p);
static void vala_semantic_analyzer_find_base_class_property (ValaSemanticAnalyzer* self, ValaProperty* prop, ValaClass* cl);
static void vala_semantic_analyzer_find_base_interface_property (ValaSemanticAnalyzer* self, ValaProperty* prop, ValaClass* cl);
static void vala_semantic_analyzer_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_semantic_analyzer_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_semantic_analyzer_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_semantic_analyzer_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_semantic_analyzer_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_semantic_analyzer_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_semantic_analyzer_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_semantic_analyzer_real_visit_named_argument (ValaCodeVisitor* base, ValaNamedArgument* n);
static void vala_semantic_analyzer_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_semantic_analyzer_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_semantic_analyzer_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl);
static void vala_semantic_analyzer_real_visit_begin_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list);
static void vala_semantic_analyzer_real_visit_end_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list);
static void vala_semantic_analyzer_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt);
static void vala_semantic_analyzer_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt);
static void vala_semantic_analyzer_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt);
static void vala_semantic_analyzer_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt);
static void vala_semantic_analyzer_real_visit_begin_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt);
static void vala_semantic_analyzer_real_visit_end_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt);
static void vala_semantic_analyzer_real_visit_lock_statement (ValaCodeVisitor* base, ValaLockStatement* stmt);
static void vala_semantic_analyzer_real_visit_begin_array_creation_expression (ValaCodeVisitor* base, ValaArrayCreationExpression* expr);
static void vala_semantic_analyzer_real_visit_end_array_creation_expression (ValaCodeVisitor* base, ValaArrayCreationExpression* expr);
static void vala_semantic_analyzer_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr);
static void vala_semantic_analyzer_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr);
static void vala_semantic_analyzer_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr);
static void vala_semantic_analyzer_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr);
static void vala_semantic_analyzer_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr);
static void vala_semantic_analyzer_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr);
static void vala_semantic_analyzer_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr);
static ValaTypeReference* vala_semantic_analyzer_get_static_type_for_node (ValaSemanticAnalyzer* self, ValaCodeNode* node);
static void vala_semantic_analyzer_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr);
static ValaDataType* vala_semantic_analyzer_find_parent_type (ValaSemanticAnalyzer* self, ValaSymbol* sym);
static void vala_semantic_analyzer_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr);
static gboolean vala_semantic_analyzer_is_type_compatible (ValaSemanticAnalyzer* self, ValaTypeReference* expression_type, ValaTypeReference* expected_type);
static void vala_semantic_analyzer_real_visit_begin_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static gboolean vala_semantic_analyzer_check_arguments (ValaSemanticAnalyzer* self, ValaExpression* expr, ValaSymbol* msym, GList* params, GList* args);
static void vala_semantic_analyzer_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static void vala_semantic_analyzer_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr);
static void vala_semantic_analyzer_real_visit_base_access (ValaCodeVisitor* base, ValaBaseAccess* expr);
static void vala_semantic_analyzer_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr);
static void vala_semantic_analyzer_real_visit_end_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr);
static void vala_semantic_analyzer_real_visit_typeof_expression (ValaCodeVisitor* base, ValaTypeofExpression* expr);
static gboolean vala_semantic_analyzer_is_numeric_type (ValaSemanticAnalyzer* self, ValaTypeReference* type);
static gboolean vala_semantic_analyzer_is_integer_type (ValaSemanticAnalyzer* self, ValaTypeReference* type);
static void vala_semantic_analyzer_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr);
static ValaMemberAccess* vala_semantic_analyzer_find_member_access (ValaSemanticAnalyzer* self, ValaExpression* expr);
static void vala_semantic_analyzer_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr);
static void vala_semantic_analyzer_real_visit_pointer_indirection (ValaCodeVisitor* base, ValaPointerIndirection* expr);
static void vala_semantic_analyzer_real_visit_addressof_expression (ValaCodeVisitor* base, ValaAddressofExpression* expr);
static gboolean vala_semantic_analyzer_check_binary_type (ValaSemanticAnalyzer* self, ValaBinaryExpression* expr, char* operation);
static ValaTypeReference* vala_semantic_analyzer_get_arithmetic_result_type (ValaSemanticAnalyzer* self, ValaTypeReference* left_type, ValaTypeReference* right_type);
static void vala_semantic_analyzer_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr);
static void vala_semantic_analyzer_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr);
static void vala_semantic_analyzer_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr);
static char* vala_semantic_analyzer_get_lambda_name (ValaSemanticAnalyzer* self);
static ValaMethod* vala_semantic_analyzer_find_current_method (ValaSemanticAnalyzer* self);
static gboolean vala_semantic_analyzer_is_in_constructor (ValaSemanticAnalyzer* self);
static void vala_semantic_analyzer_real_visit_begin_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l);
static void vala_semantic_analyzer_real_visit_begin_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static void vala_semantic_analyzer_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static gpointer vala_semantic_analyzer_parent_class = NULL;
static void vala_semantic_analyzer_dispose (GObject * obj);


ValaSemanticAnalyzer* vala_semantic_analyzer_new (gboolean manage_memory)
{
	GParameter * __params = g_new0 (GParameter, 1);
	GParameter * __params_it = __params;
	(__params_it->name = "memory-management", g_value_init (&__params_it->value, G_TYPE_BOOLEAN), g_value_set_boolean (&__params_it->value, manage_memory), __params_it++);
	ValaSemanticAnalyzer * self = g_object_newv (VALA_TYPE_SEMANTIC_ANALYZER, __params_it - __params, __params);
	return self;
}


/**
 * Analyze and check code in the specified context.
 *
 * @param context a code context
 */
void vala_semantic_analyzer_analyze (ValaSemanticAnalyzer* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	ValaSymbol* __temp0 = NULL;
	self->priv->root_symbol = (__temp0 = vala_code_context_get_root (context), (self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL))), __temp0);
	ValaTypeReference* __temp1 = NULL;
	self->priv->bool_type = (__temp1 = vala_type_reference_new (), (self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL))), __temp1);
	ValaSymbol* __temp2 = NULL;
	(vala_type_reference_set_data_type (self->priv->bool_type, VALA_DATA_TYPE (vala_symbol_get_node ((__temp2 = vala_symbol_lookup (self->priv->root_symbol, "bool"))))), vala_type_reference_get_data_type (self->priv->bool_type));
	(__temp2 == NULL ? NULL : (__temp2 = (g_object_unref (__temp2), NULL)));
	ValaTypeReference* __temp3 = NULL;
	self->priv->string_type = (__temp3 = vala_type_reference_new (), (self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL))), __temp3);
	ValaSymbol* __temp4 = NULL;
	(vala_type_reference_set_data_type (self->priv->string_type, VALA_DATA_TYPE (vala_symbol_get_node ((__temp4 = vala_symbol_lookup (self->priv->root_symbol, "string"))))), vala_type_reference_get_data_type (self->priv->string_type));
	(__temp4 == NULL ? NULL : (__temp4 = (g_object_unref (__temp4), NULL)));
	ValaDataType* __temp7 = NULL;
	ValaDataType* __temp6 = NULL;
	ValaSymbol* __temp5 = NULL;
	self->priv->pointer_type = (__temp7 = (__temp6 = VALA_DATA_TYPE (vala_symbol_get_node ((__temp5 = vala_symbol_lookup (self->priv->root_symbol, "pointer")))), (__temp6 == NULL ? NULL : g_object_ref (__temp6))), (self->priv->pointer_type == NULL ? NULL : (self->priv->pointer_type = (g_object_unref (self->priv->pointer_type), NULL))), __temp7);
	(__temp5 == NULL ? NULL : (__temp5 = (g_object_unref (__temp5), NULL)));
	ValaTypeReference* __temp8 = NULL;
	self->priv->int_type = (__temp8 = vala_type_reference_new (), (self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL))), __temp8);
	ValaSymbol* __temp9 = NULL;
	(vala_type_reference_set_data_type (self->priv->int_type, VALA_DATA_TYPE (vala_symbol_get_node ((__temp9 = vala_symbol_lookup (self->priv->root_symbol, "int"))))), vala_type_reference_get_data_type (self->priv->int_type));
	(__temp9 == NULL ? NULL : (__temp9 = (g_object_unref (__temp9), NULL)));
	/* TODO: don't require GLib namespace in semantic analyzer*/
	ValaSymbol* glib_ns = vala_symbol_lookup (self->priv->root_symbol, "GLib");
	if (glib_ns != NULL) {
		ValaDataType* __temp12 = NULL;
		ValaDataType* __temp11 = NULL;
		ValaSymbol* __temp10 = NULL;
		self->priv->initially_unowned_type = (__temp12 = (__temp11 = VALA_DATA_TYPE (vala_symbol_get_node ((__temp10 = vala_symbol_lookup (glib_ns, "InitiallyUnowned")))), (__temp11 == NULL ? NULL : g_object_ref (__temp11))), (self->priv->initially_unowned_type == NULL ? NULL : (self->priv->initially_unowned_type = (g_object_unref (self->priv->initially_unowned_type), NULL))), __temp12);
		(__temp10 == NULL ? NULL : (__temp10 = (g_object_unref (__temp10), NULL)));
		ValaTypeReference* __temp13 = NULL;
		self->priv->type_type = (__temp13 = vala_type_reference_new (), (self->priv->type_type == NULL ? NULL : (self->priv->type_type = (g_object_unref (self->priv->type_type), NULL))), __temp13);
		ValaSymbol* __temp14 = NULL;
		(vala_type_reference_set_data_type (self->priv->type_type, VALA_DATA_TYPE (vala_symbol_get_node ((__temp14 = vala_symbol_lookup (glib_ns, "Type"))))), vala_type_reference_get_data_type (self->priv->type_type));
		(__temp14 == NULL ? NULL : (__temp14 = (g_object_unref (__temp14), NULL)));
	}
	ValaSymbol* __temp16 = NULL;
	ValaSymbol* __temp15 = NULL;
	self->priv->current_symbol = (__temp16 = (__temp15 = self->priv->root_symbol, (__temp15 == NULL ? NULL : g_object_ref (__temp15))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp16);
	vala_code_context_accept (context, VALA_CODE_VISITOR (self));
	(glib_ns == NULL ? NULL : (glib_ns = (g_object_unref (glib_ns), NULL)));
}


static void vala_semantic_analyzer_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (file));
	ValaSourceFile* __temp17 = NULL;
	self->priv->current_source_file = (__temp17 = g_object_ref (file), (self->priv->current_source_file == NULL ? NULL : (self->priv->current_source_file = (g_object_unref (self->priv->current_source_file), NULL))), __temp17);
	GList* __temp18 = NULL;
	self->priv->current_using_directives = (__temp18 = vala_source_file_get_using_directives (file), (self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL))), __temp18);
	self->priv->next_lambda_id = 0;
}


static void vala_semantic_analyzer_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* file)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (file));
	GList* __temp19 = NULL;
	self->priv->current_using_directives = (__temp19 = NULL, (self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL))), __temp19);
}


static void vala_semantic_analyzer_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	ValaSymbol* __temp21 = NULL;
	ValaSymbol* __temp20 = NULL;
	self->priv->current_symbol = (__temp21 = (__temp20 = vala_code_node_get_symbol (VALA_CODE_NODE (ns)), (__temp20 == NULL ? NULL : g_object_ref (__temp20))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp21);
}


static void vala_semantic_analyzer_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	ValaSymbol* __temp23 = NULL;
	ValaSymbol* __temp22 = NULL;
	self->priv->current_symbol = (__temp23 = (__temp22 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp22 == NULL ? NULL : g_object_ref (__temp22))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp23);
}


static void vala_semantic_analyzer_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* __temp25 = NULL;
	ValaSymbol* __temp24 = NULL;
	self->priv->current_symbol = (__temp25 = (__temp24 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp24 == NULL ? NULL : g_object_ref (__temp24))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp25);
	ValaClass* __temp26 = NULL;
	self->priv->current_class = (__temp26 = g_object_ref (cl), (self->priv->current_class == NULL ? NULL : (self->priv->current_class = (g_object_unref (self->priv->current_class), NULL))), __temp26);
	if (vala_class_get_base_class (cl) != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_class_get_base_class (cl))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL);
	}
	{
		GList* __temp27 = NULL;
		__temp27 = vala_class_get_base_types (cl);
		GList* base_type_reference_it;
		for (base_type_reference_it = __temp27; base_type_reference_it != NULL; base_type_reference_it = base_type_reference_it->next) {
			ValaTypeReference* base_type_reference = base_type_reference_it->data;
			{
				vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (base_type_reference))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL);
			}
		}
		(__temp27 == NULL ? NULL : (__temp27 = (g_list_free (__temp27), NULL)));
	}
}


static GList* vala_semantic_analyzer_get_all_prerequisites (ValaSemanticAnalyzer* self, ValaInterface* iface)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_INTERFACE (iface), NULL);
	GList* ret = NULL;
	{
		GList* __temp30 = NULL;
		__temp30 = vala_interface_get_prerequisites (iface);
		GList* prereq_it;
		for (prereq_it = __temp30; prereq_it != NULL; prereq_it = prereq_it->next) {
			ValaTypeReference* prereq = prereq_it->data;
			{
				ValaDataType* __temp28 = NULL;
				ValaDataType* type = (__temp28 = vala_type_reference_get_data_type (prereq), (__temp28 == NULL ? NULL : g_object_ref (__temp28)));
				/* skip on previous errors */
				if (type == NULL) {
					continue;
				}
				ValaDataType* __temp29 = NULL;
				ret = g_list_prepend (ret, (__temp29 = type, (__temp29 == NULL ? NULL : g_object_ref (__temp29))));
				if (VALA_IS_INTERFACE (type)) {
					ret = g_list_concat (ret, vala_semantic_analyzer_get_all_prerequisites (self, VALA_INTERFACE (type)));
				}
				(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
			}
		}
		(__temp30 == NULL ? NULL : (__temp30 = (g_list_free (__temp30), NULL)));
	}
	return ret = g_list_reverse (ret);
}


static gboolean vala_semantic_analyzer_class_is_a (ValaSemanticAnalyzer* self, ValaClass* cl, ValaDataType* t)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_CLASS (cl), FALSE);
	g_return_val_if_fail (VALA_IS_DATA_TYPE (t), FALSE);
	if (VALA_DATA_TYPE (cl) == t) {
		return TRUE;
	}
	{
		GList* __temp35 = NULL;
		__temp35 = vala_class_get_base_types (cl);
		GList* base_type_it;
		for (base_type_it = __temp35; base_type_it != NULL; base_type_it = base_type_it->next) {
			ValaTypeReference* base_type = base_type_it->data;
			{
				if (VALA_IS_CLASS (vala_type_reference_get_data_type (base_type))) {
					if (vala_semantic_analyzer_class_is_a (self, VALA_CLASS (vala_type_reference_get_data_type (base_type)), t)) {
						return TRUE;
					}
				} else {
					if (vala_type_reference_get_data_type (base_type) == t) {
						return TRUE;
					}
				}
			}
		}
		(__temp35 == NULL ? NULL : (__temp35 = (g_list_free (__temp35), NULL)));
	}
	return FALSE;
}


static void vala_semantic_analyzer_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	/* gather all prerequisites */
	GList* prerequisites = NULL;
	{
		GList* __temp37 = NULL;
		__temp37 = vala_class_get_base_types (cl);
		GList* base_type_it;
		for (base_type_it = __temp37; base_type_it != NULL; base_type_it = base_type_it->next) {
			ValaTypeReference* base_type = base_type_it->data;
			{
				if (VALA_IS_INTERFACE (vala_type_reference_get_data_type (base_type))) {
					prerequisites = g_list_concat (prerequisites, vala_semantic_analyzer_get_all_prerequisites (self, VALA_INTERFACE (vala_type_reference_get_data_type (base_type))));
				}
			}
		}
		(__temp37 == NULL ? NULL : (__temp37 = (g_list_free (__temp37), NULL)));
	}
	/* check whether all prerequisites are met */
	GList* missing_prereqs = NULL;
	{
		GList* __temp38 = NULL;
		__temp38 = prerequisites;
		GList* prereq_it;
		for (prereq_it = __temp38; prereq_it != NULL; prereq_it = prereq_it->next) {
			ValaDataType* prereq = prereq_it->data;
			{
				if (!vala_semantic_analyzer_class_is_a (self, cl, prereq)) {
					missing_prereqs = g_list_prepend (missing_prereqs, vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (prereq))));
				}
			}
		}
	}
	/* report any missing prerequisites */
	if (missing_prereqs != NULL) {
		(vala_code_node_set_error (VALA_CODE_NODE (cl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (cl)));
		char* __temp39 = NULL;
		char* __temp40 = NULL;
		char* error_string = (__temp40 = g_strdup_printf ("%s: some prerequisites (", (__temp39 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cl))))), (__temp39 == NULL ? NULL : (__temp39 = (g_free (__temp39), NULL))), __temp40);
		gboolean first = TRUE;
		{
			GList* __temp43 = NULL;
			__temp43 = missing_prereqs;
			GList* s_it;
			for (s_it = __temp43; s_it != NULL; s_it = s_it->next) {
				char* s = s_it->data;
				{
					if (first) {
						char* __temp41 = NULL;
						error_string = (__temp41 = g_strdup_printf ("%s`%s'", error_string, s), (error_string == NULL ? NULL : (error_string = (g_free (error_string), NULL))), __temp41);
						first = FALSE;
					} else {
						char* __temp42 = NULL;
						error_string = (__temp42 = g_strdup_printf ("%s, `%s'", error_string, s), (error_string == NULL ? NULL : (error_string = (g_free (error_string), NULL))), __temp42);
					}
				}
			}
		}
		char* __temp44 = NULL;
		error_string = (__temp44 = g_strconcat (error_string, (") are not met"), NULL), (error_string == NULL ? NULL : (error_string = (g_free (error_string), NULL))), __temp44);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (cl)), error_string);
		(error_string == NULL ? NULL : (error_string = (g_free (error_string), NULL)));
	}
	{
		GList* __temp50 = NULL;
		__temp50 = vala_class_get_base_types (cl);
		GList* base_type_it;
		for (base_type_it = __temp50; base_type_it != NULL; base_type_it = base_type_it->next) {
			ValaTypeReference* base_type = base_type_it->data;
			{
				/* all abstract symbols defined in base types have to be at least defined (or implemented) also in this type */
				if (VALA_IS_INTERFACE (vala_type_reference_get_data_type (base_type))) {
					ValaInterface* __temp45 = NULL;
					ValaInterface* iface = (__temp45 = VALA_INTERFACE (vala_type_reference_get_data_type (base_type)), (__temp45 == NULL ? NULL : g_object_ref (__temp45)));
					{
						GList* __temp49 = NULL;
						__temp49 = vala_interface_get_methods (iface);
						GList* m_it;
						for (m_it = __temp49; m_it != NULL; m_it = m_it->next) {
							ValaMethod* m = m_it->data;
							{
								/* We do not need to do expensive equality checking here since this is done
								 * already. We only need to guarantee the symbols are present.
								 
								 check methods */
								if (vala_method_get_is_abstract (m)) {
									ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (cl)), vala_method_get_name (m));
									if (sym == NULL || !(VALA_IS_METHOD (vala_symbol_get_node (sym))) || vala_method_get_base_interface_method ((VALA_METHOD (vala_symbol_get_node (sym)))) != m) {
										(vala_code_node_set_error (VALA_CODE_NODE (cl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (cl)));
										char* __temp48 = NULL;
										char* __temp47 = NULL;
										char* __temp46 = NULL;
										vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (cl)), (__temp48 = g_strdup_printf ("`%s' does not implement interface method `%s'", (__temp46 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cl)))), (__temp47 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))));
										(__temp48 == NULL ? NULL : (__temp48 = (g_free (__temp48), NULL)));
										(__temp47 == NULL ? NULL : (__temp47 = (g_free (__temp47), NULL)));
										(__temp46 == NULL ? NULL : (__temp46 = (g_free (__temp46), NULL)));
									}
									(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
								}
							}
						}
						(__temp49 == NULL ? NULL : (__temp49 = (g_list_free (__temp49), NULL)));
					}
					(iface == NULL ? NULL : (iface = (g_object_unref (iface), NULL)));
				}
			}
		}
		(__temp50 == NULL ? NULL : (__temp50 = (g_list_free (__temp50), NULL)));
	}
	/* all abstract symbols defined in base classes have to be implemented in non-abstract classes
	 * VAPI classes don't have to specify overridden methods
	 */
	if (!vala_class_get_is_abstract (cl) && !vala_source_file_get_pkg (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (cl))))) {
		ValaClass* __temp51 = NULL;
		ValaClass* base_class = (__temp51 = vala_class_get_base_class (cl), (__temp51 == NULL ? NULL : g_object_ref (__temp51)));
		while (base_class != NULL && vala_class_get_is_abstract (base_class)) {
			{
				GList* __temp55 = NULL;
				__temp55 = vala_class_get_methods (base_class);
				GList* m_it;
				for (m_it = __temp55; m_it != NULL; m_it = m_it->next) {
					ValaMethod* m = m_it->data;
					{
						if (vala_method_get_is_abstract (m)) {
							ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (cl)), vala_method_get_name (m));
							if (sym == NULL || !(VALA_IS_METHOD (vala_symbol_get_node (sym))) || vala_method_get_base_method ((VALA_METHOD (vala_symbol_get_node (sym)))) != m) {
								(vala_code_node_set_error (VALA_CODE_NODE (cl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (cl)));
								char* __temp54 = NULL;
								char* __temp53 = NULL;
								char* __temp52 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (cl)), (__temp54 = g_strdup_printf ("`%s' does not implement abstract method `%s'", (__temp52 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cl)))), (__temp53 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))));
								(__temp54 == NULL ? NULL : (__temp54 = (g_free (__temp54), NULL)));
								(__temp53 == NULL ? NULL : (__temp53 = (g_free (__temp53), NULL)));
								(__temp52 == NULL ? NULL : (__temp52 = (g_free (__temp52), NULL)));
							}
							(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
						}
					}
				}
				(__temp55 == NULL ? NULL : (__temp55 = (g_list_free (__temp55), NULL)));
			}
			ValaClass* __temp57 = NULL;
			ValaClass* __temp56 = NULL;
			base_class = (__temp57 = (__temp56 = vala_class_get_base_class (base_class), (__temp56 == NULL ? NULL : g_object_ref (__temp56))), (base_class == NULL ? NULL : (base_class = (g_object_unref (base_class), NULL))), __temp57);
		}
		(base_class == NULL ? NULL : (base_class = (g_object_unref (base_class), NULL)));
	}
	ValaSymbol* __temp59 = NULL;
	ValaSymbol* __temp58 = NULL;
	self->priv->current_symbol = (__temp59 = (__temp58 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp58 == NULL ? NULL : g_object_ref (__temp58))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp59);
	ValaClass* __temp60 = NULL;
	self->priv->current_class = (__temp60 = NULL, (self->priv->current_class == NULL ? NULL : (self->priv->current_class = (g_object_unref (self->priv->current_class), NULL))), __temp60);
	(prerequisites == NULL ? NULL : (prerequisites = (g_list_foreach (prerequisites, (GFunc) g_object_unref, NULL), g_list_free (prerequisites), NULL)));
	(missing_prereqs == NULL ? NULL : (missing_prereqs = (g_list_foreach (missing_prereqs, (GFunc) g_free, NULL), g_list_free (missing_prereqs), NULL)));
}


static void vala_semantic_analyzer_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaSymbol* __temp62 = NULL;
	ValaSymbol* __temp61 = NULL;
	self->priv->current_symbol = (__temp62 = (__temp61 = vala_code_node_get_symbol (VALA_CODE_NODE (st)), (__temp61 == NULL ? NULL : g_object_ref (__temp61))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp62);
	ValaStruct* __temp63 = NULL;
	self->priv->current_struct = (__temp63 = g_object_ref (st), (self->priv->current_struct == NULL ? NULL : (self->priv->current_struct = (g_object_unref (self->priv->current_struct), NULL))), __temp63);
}


static void vala_semantic_analyzer_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaSymbol* __temp65 = NULL;
	ValaSymbol* __temp64 = NULL;
	self->priv->current_symbol = (__temp65 = (__temp64 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp64 == NULL ? NULL : g_object_ref (__temp64))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp65);
	ValaStruct* __temp66 = NULL;
	self->priv->current_struct = (__temp66 = NULL, (self->priv->current_struct == NULL ? NULL : (self->priv->current_struct = (g_object_unref (self->priv->current_struct), NULL))), __temp66);
}


static void vala_semantic_analyzer_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	ValaSymbol* __temp68 = NULL;
	ValaSymbol* __temp67 = NULL;
	self->priv->current_symbol = (__temp68 = (__temp67 = vala_code_node_get_symbol (VALA_CODE_NODE (iface)), (__temp67 == NULL ? NULL : g_object_ref (__temp67))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp68);
	{
		GList* __temp69 = NULL;
		__temp69 = vala_interface_get_prerequisites (iface);
		GList* prerequisite_reference_it;
		for (prerequisite_reference_it = __temp69; prerequisite_reference_it != NULL; prerequisite_reference_it = prerequisite_reference_it->next) {
			ValaTypeReference* prerequisite_reference = prerequisite_reference_it->data;
			{
				vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (prerequisite_reference))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL);
			}
		}
		(__temp69 == NULL ? NULL : (__temp69 = (g_list_free (__temp69), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	/* check prerequisites */
	ValaClass* prereq_class = NULL;
	{
		GList* __temp77 = NULL;
		__temp77 = vala_interface_get_prerequisites (iface);
		GList* prereq_it;
		for (prereq_it = __temp77; prereq_it != NULL; prereq_it = prereq_it->next) {
			ValaTypeReference* prereq = prereq_it->data;
			{
				ValaDataType* __temp70 = NULL;
				ValaDataType* class_or_interface = (__temp70 = vala_type_reference_get_data_type (prereq), (__temp70 == NULL ? NULL : g_object_ref (__temp70)));
				/* skip on previous errors */
				if (class_or_interface == NULL) {
					(vala_code_node_set_error (VALA_CODE_NODE (iface), TRUE), vala_code_node_get_error (VALA_CODE_NODE (iface)));
					continue;
				}
				/* interfaces are not allowed to have multiple instantiable prerequisites */
				if (VALA_IS_CLASS (class_or_interface)) {
					if (prereq_class != NULL) {
						(vala_code_node_set_error (VALA_CODE_NODE (iface), TRUE), vala_code_node_get_error (VALA_CODE_NODE (iface)));
						char* __temp74 = NULL;
						char* __temp73 = NULL;
						char* __temp72 = NULL;
						char* __temp71 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (iface)), (__temp74 = g_strdup_printf ("%s: Interfaces cannot have multiple instantiable prerequisites (`%s' and `%s')", (__temp71 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (iface)))), (__temp72 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (class_or_interface)))), (__temp73 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (prereq_class)))))));
						(__temp74 == NULL ? NULL : (__temp74 = (g_free (__temp74), NULL)));
						(__temp73 == NULL ? NULL : (__temp73 = (g_free (__temp73), NULL)));
						(__temp72 == NULL ? NULL : (__temp72 = (g_free (__temp72), NULL)));
						(__temp71 == NULL ? NULL : (__temp71 = (g_free (__temp71), NULL)));
						(class_or_interface == NULL ? NULL : (class_or_interface = (g_object_unref (class_or_interface), NULL)));
						(prereq_class == NULL ? NULL : (prereq_class = (g_object_unref (prereq_class), NULL)));
						return;
					}
					ValaClass* __temp76 = NULL;
					ValaClass* __temp75 = NULL;
					prereq_class = (__temp76 = (__temp75 = VALA_CLASS (class_or_interface), (__temp75 == NULL ? NULL : g_object_ref (__temp75))), (prereq_class == NULL ? NULL : (prereq_class = (g_object_unref (prereq_class), NULL))), __temp76);
				}
				(class_or_interface == NULL ? NULL : (class_or_interface = (g_object_unref (class_or_interface), NULL)));
			}
		}
		(__temp77 == NULL ? NULL : (__temp77 = (g_list_free (__temp77), NULL)));
	}
	ValaSymbol* __temp79 = NULL;
	ValaSymbol* __temp78 = NULL;
	self->priv->current_symbol = (__temp79 = (__temp78 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp78 == NULL ? NULL : g_object_ref (__temp78))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp79);
	(prereq_class == NULL ? NULL : (prereq_class = (g_object_unref (prereq_class), NULL)));
}


static void vala_semantic_analyzer_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONSTANT (c));
	if (!vala_source_file_get_pkg (self->priv->current_source_file)) {
		if (vala_constant_get_initializer (c) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (c), TRUE), vala_code_node_get_error (VALA_CODE_NODE (c)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (c)), "A const field requires a initializer to be provided");
		}
	}
}


static void vala_semantic_analyzer_real_visit_field (ValaCodeVisitor* base, ValaField* f)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FIELD (f));
	if (f->access != VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
		if (vala_type_reference_get_data_type (vala_field_get_type_reference (f)) != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_field_get_type_reference (f)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
		}
	} else {
		if (vala_type_reference_get_data_type (vala_field_get_type_reference (f)) != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_field_get_type_reference (f)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
		}
	}
}


static void vala_semantic_analyzer_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp81 = NULL;
	ValaSymbol* __temp80 = NULL;
	self->priv->current_symbol = (__temp81 = (__temp80 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp80 == NULL ? NULL : g_object_ref (__temp80))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp81);
	ValaTypeReference* __temp83 = NULL;
	ValaTypeReference* __temp82 = NULL;
	self->priv->current_return_type = (__temp83 = (__temp82 = vala_method_get_return_type (m), (__temp82 == NULL ? NULL : g_object_ref (__temp82))), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp83);
	ValaAttribute* init_attr = vala_code_node_get_attribute (VALA_CODE_NODE (m), "ModuleInit");
	if (init_attr != NULL) {
		(vala_code_context_set_module_init_method (vala_source_file_get_context (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (m)))), m), vala_code_context_get_module_init_method (vala_source_file_get_context (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (m))))));
	}
	if (vala_type_reference_get_data_type (vala_method_get_return_type (m)) != NULL) {
		/* is null if it is void or a reference to a type parameter */
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_method_get_return_type (m)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
	}
	(init_attr == NULL ? NULL : (init_attr = (g_object_unref (init_attr), NULL)));
}


static void vala_semantic_analyzer_find_base_class_method (ValaSemanticAnalyzer* self, ValaMethod* m, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	g_return_if_fail (VALA_IS_METHOD (m));
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (cl)), vala_method_get_name (m));
	if (sym != NULL && VALA_IS_METHOD (vala_symbol_get_node (sym))) {
		ValaMethod* __temp84 = NULL;
		ValaMethod* base_method = (__temp84 = VALA_METHOD (vala_symbol_get_node (sym)), (__temp84 == NULL ? NULL : g_object_ref (__temp84)));
		if (vala_method_get_is_abstract (base_method) || vala_method_get_is_virtual (base_method)) {
			if (!vala_method_equals (m, base_method)) {
				(vala_code_node_set_error (VALA_CODE_NODE (m), TRUE), vala_code_node_get_error (VALA_CODE_NODE (m)));
				char* __temp87 = NULL;
				char* __temp86 = NULL;
				char* __temp85 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp87 = g_strdup_printf ("Return type and/or parameters of overriding method `%s' do not match overridden method `%s'.", (__temp85 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp86 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (base_method)))))));
				(__temp87 == NULL ? NULL : (__temp87 = (g_free (__temp87), NULL)));
				(__temp86 == NULL ? NULL : (__temp86 = (g_free (__temp86), NULL)));
				(__temp85 == NULL ? NULL : (__temp85 = (g_free (__temp85), NULL)));
				(base_method == NULL ? NULL : (base_method = (g_object_unref (base_method), NULL)));
				(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
				return;
			}
			(vala_method_set_base_method (m, base_method), vala_method_get_base_method (m));
			(base_method == NULL ? NULL : (base_method = (g_object_unref (base_method), NULL)));
			(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
			return;
		}
		(base_method == NULL ? NULL : (base_method = (g_object_unref (base_method), NULL)));
	}
	if (vala_class_get_base_class (cl) != NULL) {
		vala_semantic_analyzer_find_base_class_method (self, m, vala_class_get_base_class (cl));
	}
	(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
}


static void vala_semantic_analyzer_find_base_interface_method (ValaSemanticAnalyzer* self, ValaMethod* m, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	g_return_if_fail (VALA_IS_METHOD (m));
	g_return_if_fail (VALA_IS_CLASS (cl));
	{
		GList* __temp92 = NULL;
		__temp92 = vala_class_get_base_types (cl);
		GList* type_it;
		for (type_it = __temp92; type_it != NULL; type_it = type_it->next) {
			ValaTypeReference* type = type_it->data;
			{
				/* FIXME report error if multiple possible base methods are found*/
				if (VALA_IS_INTERFACE (vala_type_reference_get_data_type (type))) {
					ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (type))), vala_method_get_name (m));
					if (sym != NULL && VALA_IS_METHOD (vala_symbol_get_node (sym))) {
						ValaMethod* __temp88 = NULL;
						ValaMethod* base_method = (__temp88 = VALA_METHOD (vala_symbol_get_node (sym)), (__temp88 == NULL ? NULL : g_object_ref (__temp88)));
						if (vala_method_get_is_abstract (base_method)) {
							if (!vala_method_equals (m, base_method)) {
								(vala_code_node_set_error (VALA_CODE_NODE (m), TRUE), vala_code_node_get_error (VALA_CODE_NODE (m)));
								char* __temp91 = NULL;
								char* __temp90 = NULL;
								char* __temp89 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp91 = g_strdup_printf ("Return type and/or parameters of overriding method `%s' do not match overridden method `%s'.", (__temp89 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp90 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (base_method)))))));
								(__temp91 == NULL ? NULL : (__temp91 = (g_free (__temp91), NULL)));
								(__temp90 == NULL ? NULL : (__temp90 = (g_free (__temp90), NULL)));
								(__temp89 == NULL ? NULL : (__temp89 = (g_free (__temp89), NULL)));
								(base_method == NULL ? NULL : (base_method = (g_object_unref (base_method), NULL)));
								(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
								return;
							}
							(vala_method_set_base_interface_method (m, base_method), vala_method_get_base_interface_method (m));
							(base_method == NULL ? NULL : (base_method = (g_object_unref (base_method), NULL)));
							(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
							return;
						}
						(base_method == NULL ? NULL : (base_method = (g_object_unref (base_method), NULL)));
					}
					(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
				}
			}
		}
		(__temp92 == NULL ? NULL : (__temp92 = (g_list_free (__temp92), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp94 = NULL;
	ValaSymbol* __temp93 = NULL;
	self->priv->current_symbol = (__temp94 = (__temp93 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp93 == NULL ? NULL : g_object_ref (__temp93))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp94);
	ValaTypeReference* __temp95 = NULL;
	self->priv->current_return_type = (__temp95 = NULL, (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp95);
	if (vala_symbol_get_parent_symbol (self->priv->current_symbol) != NULL && VALA_IS_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (self->priv->current_symbol)))) {
		/* lambda expressions produce nested methods */
		ValaMethod* __temp96 = NULL;
		ValaMethod* up_method = (__temp96 = VALA_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (self->priv->current_symbol))), (__temp96 == NULL ? NULL : g_object_ref (__temp96)));
		ValaTypeReference* __temp98 = NULL;
		ValaTypeReference* __temp97 = NULL;
		self->priv->current_return_type = (__temp98 = (__temp97 = vala_method_get_return_type (up_method), (__temp97 == NULL ? NULL : g_object_ref (__temp97))), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp98);
		(up_method == NULL ? NULL : (up_method = (g_object_unref (up_method), NULL)));
	}
	if (VALA_IS_CLASS (vala_symbol_get_node (self->priv->current_symbol))) {
		if (!(VALA_IS_CREATION_METHOD (m))) {
			vala_semantic_analyzer_find_base_interface_method (self, m, VALA_CLASS (vala_symbol_get_node (self->priv->current_symbol)));
			if (vala_method_get_is_virtual (m) || vala_method_get_overrides (m)) {
				vala_semantic_analyzer_find_base_class_method (self, m, VALA_CLASS (vala_symbol_get_node (self->priv->current_symbol)));
				if (vala_method_get_base_method (m) == NULL) {
					char* __temp100 = NULL;
					char* __temp99 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp100 = g_strdup_printf ("%s: no suitable method found to override", (__temp99 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))));
					(__temp100 == NULL ? NULL : (__temp100 = (g_free (__temp100), NULL)));
					(__temp99 == NULL ? NULL : (__temp99 = (g_free (__temp99), NULL)));
				}
			}
		}
	} else {
		if (VALA_IS_STRUCT (vala_symbol_get_node (self->priv->current_symbol))) {
			if (vala_method_get_is_abstract (m) || vala_method_get_is_virtual (m) || vala_method_get_overrides (m)) {
				char* __temp102 = NULL;
				char* __temp101 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp102 = g_strdup_printf ("A struct member `%s' cannot be marked as override, virtual, or abstract", (__temp101 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))));
				(__temp102 == NULL ? NULL : (__temp102 = (g_free (__temp102), NULL)));
				(__temp101 == NULL ? NULL : (__temp101 = (g_free (__temp101), NULL)));
				return;
			}
		}
	}
}


static void vala_semantic_analyzer_real_visit_begin_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CREATION_METHOD (m));
	ValaTypeReference* __temp103 = NULL;
	(vala_method_set_return_type (VALA_METHOD (m), (__temp103 = vala_type_reference_new ())), vala_method_get_return_type (VALA_METHOD (m)));
	(__temp103 == NULL ? NULL : (__temp103 = (g_object_unref (__temp103), NULL)));
	(vala_type_reference_set_data_type (vala_method_get_return_type (VALA_METHOD (m)), VALA_DATA_TYPE (vala_symbol_get_node (self->priv->current_symbol))), vala_type_reference_get_data_type (vala_method_get_return_type (VALA_METHOD (m))));
	(vala_type_reference_set_transfers_ownership (vala_method_get_return_type (VALA_METHOD (m)), TRUE), vala_type_reference_get_transfers_ownership (vala_method_get_return_type (VALA_METHOD (m))));
	if (VALA_IS_CLASS (vala_symbol_get_node (self->priv->current_symbol))) {
		/* check for floating reference*/
		ValaClass* __temp104 = NULL;
		ValaClass* cl = (__temp104 = VALA_CLASS (vala_symbol_get_node (self->priv->current_symbol)), (__temp104 == NULL ? NULL : g_object_ref (__temp104)));
		while (cl != NULL) {
			if (VALA_DATA_TYPE (cl) == self->priv->initially_unowned_type) {
				(vala_type_reference_set_floating_reference (vala_method_get_return_type (VALA_METHOD (m)), TRUE), vala_type_reference_get_floating_reference (vala_method_get_return_type (VALA_METHOD (m))));
				break;
			}
			ValaClass* __temp106 = NULL;
			ValaClass* __temp105 = NULL;
			cl = (__temp106 = (__temp105 = vala_class_get_base_class (cl), (__temp105 == NULL ? NULL : g_object_ref (__temp105))), (cl == NULL ? NULL : (cl = (g_object_unref (cl), NULL))), __temp106);
		}
		(cl == NULL ? NULL : (cl = (g_object_unref (cl), NULL)));
	}
	if (vala_method_get_body (VALA_METHOD (m)) != NULL) {
		(vala_statement_set_construction (VALA_STATEMENT (vala_method_get_body (VALA_METHOD (m))), TRUE), vala_statement_get_construction (VALA_STATEMENT (vala_method_get_body (VALA_METHOD (m)))));
	}
	ValaSymbol* __temp108 = NULL;
	ValaSymbol* __temp107 = NULL;
	self->priv->current_symbol = (__temp108 = (__temp107 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp107 == NULL ? NULL : g_object_ref (__temp107))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp108);
	ValaTypeReference* __temp110 = NULL;
	ValaTypeReference* __temp109 = NULL;
	self->priv->current_return_type = (__temp110 = (__temp109 = vala_method_get_return_type (VALA_METHOD (m)), (__temp109 == NULL ? NULL : g_object_ref (__temp109))), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp110);
}


static void vala_semantic_analyzer_real_visit_end_creation_method (ValaCodeVisitor* base, ValaCreationMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CREATION_METHOD (m));
	vala_code_visitor_visit_end_method (VALA_CODE_VISITOR (self), VALA_METHOD (m));
	if (vala_method_get_body (VALA_METHOD (m)) != NULL) {
		gint n_params = 0;
		{
			GList* __temp111 = NULL;
			__temp111 = vala_block_get_statements (vala_method_get_body (VALA_METHOD (m)));
			GList* stmt_it;
			for (stmt_it = __temp111; stmt_it != NULL; stmt_it = stmt_it->next) {
				ValaStatement* stmt = stmt_it->data;
				{
					gint params = vala_statement_get_number_of_set_construction_parameters (stmt);
					if (params == -1) {
						(vala_code_node_set_error (VALA_CODE_NODE (m), TRUE), vala_code_node_get_error (VALA_CODE_NODE (m)));
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "type creation methods only allow property assignment statements");
						return;
					}
					n_params = n_params + (params);
					(vala_statement_set_construction (stmt, TRUE), vala_statement_get_construction (stmt));
				}
			}
			(__temp111 == NULL ? NULL : (__temp111 = (g_list_free (__temp111), NULL)));
		}
		(vala_creation_method_set_n_construction_params (m, n_params), vala_creation_method_get_n_construction_params (m));
	}
}


static void vala_semantic_analyzer_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (p));
	if (!vala_formal_parameter_get_ellipsis (p)) {
		if (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)) != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (p)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
		}
	}
	/* special treatment for construct formal parameters used in creation methods */
	if (vala_formal_parameter_get_construct_parameter (p)) {
		if (!(VALA_IS_CREATION_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (p))))))) {
			(vala_code_node_set_error (VALA_CODE_NODE (p), TRUE), vala_code_node_get_error (VALA_CODE_NODE (p)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (p)), "construct parameters are only allowed in type creation methods");
			return;
		}
		ValaBlock* __temp112 = NULL;
		ValaBlock* method_body = (__temp112 = vala_method_get_body (VALA_METHOD ((VALA_CREATION_METHOD (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (p)))))))), (__temp112 == NULL ? NULL : g_object_ref (__temp112)));
		ValaMemberAccess* left = vala_member_access_new_simple (vala_formal_parameter_get_name (p), NULL);
		ValaMemberAccess* right = vala_member_access_new_simple (vala_formal_parameter_get_name (p), NULL);
		/* try to lookup the requested property */
		ValaSymbol* prop_sym = vala_semantic_analyzer_symbol_lookup_inherited (vala_code_node_get_symbol (VALA_CODE_NODE (self->priv->current_class)), vala_formal_parameter_get_name (p));
		if (!(VALA_IS_PROPERTY (vala_symbol_get_node (prop_sym)))) {
			(vala_code_node_set_error (VALA_CODE_NODE (p), TRUE), vala_code_node_get_error (VALA_CODE_NODE (p)));
			char* __temp114 = NULL;
			char* __temp113 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (p)), (__temp114 = g_strdup_printf ("class `%s' does not contain a property named `%s'", (__temp113 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (self->priv->current_class)))), vala_formal_parameter_get_name (p))));
			(__temp114 == NULL ? NULL : (__temp114 = (g_free (__temp114), NULL)));
			(__temp113 == NULL ? NULL : (__temp113 = (g_free (__temp113), NULL)));
			(method_body == NULL ? NULL : (method_body = (g_object_unref (method_body), NULL)));
			(left == NULL ? NULL : (left = (g_object_unref (left), NULL)));
			(right == NULL ? NULL : (right = (g_object_unref (right), NULL)));
			(prop_sym == NULL ? NULL : (prop_sym = (g_object_unref (prop_sym), NULL)));
			return;
		}
		(vala_expression_set_symbol_reference (VALA_EXPRESSION (left), prop_sym), vala_expression_get_symbol_reference (VALA_EXPRESSION (left)));
		(vala_expression_set_symbol_reference (VALA_EXPRESSION (right), vala_code_node_get_symbol (VALA_CODE_NODE (p))), vala_expression_get_symbol_reference (VALA_EXPRESSION (right)));
		ValaExpressionStatement* __temp115 = NULL;
		vala_block_add_statement (method_body, VALA_STATEMENT ((__temp115 = vala_expression_statement_new (VALA_EXPRESSION (vala_assignment_new (VALA_EXPRESSION (left), VALA_EXPRESSION (right), VALA_ASSIGNMENT_OPERATOR_SIMPLE, NULL)), NULL))));
		(__temp115 == NULL ? NULL : (__temp115 = (g_object_unref (__temp115), NULL)));
		(method_body == NULL ? NULL : (method_body = (g_object_unref (method_body), NULL)));
		(left == NULL ? NULL : (left = (g_object_unref (left), NULL)));
		(right == NULL ? NULL : (right = (g_object_unref (right), NULL)));
		(prop_sym == NULL ? NULL : (prop_sym = (g_object_unref (prop_sym), NULL)));
	}
}


static void vala_semantic_analyzer_find_base_class_property (ValaSemanticAnalyzer* self, ValaProperty* prop, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (cl)), vala_property_get_name (prop));
	if (sym != NULL && VALA_IS_PROPERTY (vala_symbol_get_node (sym))) {
		ValaProperty* __temp116 = NULL;
		ValaProperty* base_property = (__temp116 = VALA_PROPERTY (vala_symbol_get_node (sym)), (__temp116 == NULL ? NULL : g_object_ref (__temp116)));
		if (vala_property_get_is_abstract (base_property) || vala_property_get_is_virtual (base_property)) {
			if (!vala_property_equals (prop, base_property)) {
				(vala_code_node_set_error (VALA_CODE_NODE (prop), TRUE), vala_code_node_get_error (VALA_CODE_NODE (prop)));
				char* __temp119 = NULL;
				char* __temp118 = NULL;
				char* __temp117 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (prop)), (__temp119 = g_strdup_printf ("Type and/or accessors of overriding property `%s' do not match overridden property `%s'.", (__temp117 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))), (__temp118 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (base_property)))))));
				(__temp119 == NULL ? NULL : (__temp119 = (g_free (__temp119), NULL)));
				(__temp118 == NULL ? NULL : (__temp118 = (g_free (__temp118), NULL)));
				(__temp117 == NULL ? NULL : (__temp117 = (g_free (__temp117), NULL)));
				(base_property == NULL ? NULL : (base_property = (g_object_unref (base_property), NULL)));
				(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
				return;
			}
			(vala_property_set_base_property (prop, base_property), vala_property_get_base_property (prop));
			(base_property == NULL ? NULL : (base_property = (g_object_unref (base_property), NULL)));
			(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
			return;
		}
		(base_property == NULL ? NULL : (base_property = (g_object_unref (base_property), NULL)));
	}
	if (vala_class_get_base_class (cl) != NULL) {
		vala_semantic_analyzer_find_base_class_property (self, prop, vala_class_get_base_class (cl));
	}
	(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
}


static void vala_semantic_analyzer_find_base_interface_property (ValaSemanticAnalyzer* self, ValaProperty* prop, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	g_return_if_fail (VALA_IS_CLASS (cl));
	{
		GList* __temp124 = NULL;
		__temp124 = vala_class_get_base_types (cl);
		GList* type_it;
		for (type_it = __temp124; type_it != NULL; type_it = type_it->next) {
			ValaTypeReference* type = type_it->data;
			{
				/* FIXME report error if multiple possible base properties are found*/
				if (VALA_IS_INTERFACE (vala_type_reference_get_data_type (type))) {
					ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (type))), vala_property_get_name (prop));
					if (sym != NULL && VALA_IS_PROPERTY (vala_symbol_get_node (sym))) {
						ValaProperty* __temp120 = NULL;
						ValaProperty* base_property = (__temp120 = VALA_PROPERTY (vala_symbol_get_node (sym)), (__temp120 == NULL ? NULL : g_object_ref (__temp120)));
						if (vala_property_get_is_abstract (base_property)) {
							if (!vala_property_equals (prop, base_property)) {
								(vala_code_node_set_error (VALA_CODE_NODE (prop), TRUE), vala_code_node_get_error (VALA_CODE_NODE (prop)));
								char* __temp123 = NULL;
								char* __temp122 = NULL;
								char* __temp121 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (prop)), (__temp123 = g_strdup_printf ("Type and/or accessors of overriding property `%s' do not match overridden property `%s'.", (__temp121 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))), (__temp122 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (base_property)))))));
								(__temp123 == NULL ? NULL : (__temp123 = (g_free (__temp123), NULL)));
								(__temp122 == NULL ? NULL : (__temp122 = (g_free (__temp122), NULL)));
								(__temp121 == NULL ? NULL : (__temp121 = (g_free (__temp121), NULL)));
								(base_property == NULL ? NULL : (base_property = (g_object_unref (base_property), NULL)));
								(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
								return;
							}
							(vala_property_set_base_interface_property (prop, base_property), vala_property_get_base_interface_property (prop));
							(base_property == NULL ? NULL : (base_property = (g_object_unref (base_property), NULL)));
							(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
							return;
						}
						(base_property == NULL ? NULL : (base_property = (g_object_unref (base_property), NULL)));
					}
					(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
				}
			}
		}
		(__temp124 == NULL ? NULL : (__temp124 = (g_list_free (__temp124), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	if (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)) != NULL) {
		/* is null if it references a type parameter */
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_property_get_type_reference (prop)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
	if (VALA_IS_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))))) {
		ValaClass* __temp125 = NULL;
		ValaClass* cl = (__temp125 = VALA_CLASS (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (prop))))), (__temp125 == NULL ? NULL : g_object_ref (__temp125)));
		vala_semantic_analyzer_find_base_interface_property (self, prop, cl);
		if (vala_property_get_is_virtual (prop) || vala_property_get_overrides (prop)) {
			vala_semantic_analyzer_find_base_class_property (self, prop, cl);
			if (vala_property_get_base_property (prop) == NULL) {
				(vala_code_node_set_error (VALA_CODE_NODE (prop), TRUE), vala_code_node_get_error (VALA_CODE_NODE (prop)));
				char* __temp127 = NULL;
				char* __temp126 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (prop)), (__temp127 = g_strdup_printf ("%s: no suitable property found to override", (__temp126 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))))));
				(__temp127 == NULL ? NULL : (__temp127 = (g_free (__temp127), NULL)));
				(__temp126 == NULL ? NULL : (__temp126 = (g_free (__temp126), NULL)));
			}
		}
		(cl == NULL ? NULL : (cl = (g_object_unref (cl), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaProperty* __temp128 = NULL;
	ValaProperty* prop = (__temp128 = VALA_PROPERTY (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (acc))))), (__temp128 == NULL ? NULL : g_object_ref (__temp128)));
	if (vala_property_accessor_get_readable (acc)) {
		ValaTypeReference* __temp129 = NULL;
		self->priv->current_return_type = (__temp129 = g_object_ref (vala_property_get_type_reference (prop)), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp129);
	} else {
		/* void*/
		ValaTypeReference* __temp130 = NULL;
		self->priv->current_return_type = (__temp130 = vala_type_reference_new (), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp130);
	}
	(prop == NULL ? NULL : (prop = (g_object_unref (prop), NULL)));
}


static void vala_semantic_analyzer_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaTypeReference* __temp131 = NULL;
	self->priv->current_return_type = (__temp131 = NULL, (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp131);
}


static void vala_semantic_analyzer_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp133 = NULL;
	ValaSymbol* __temp132 = NULL;
	self->priv->current_symbol = (__temp133 = (__temp132 = vala_code_node_get_symbol (VALA_CODE_NODE (c)), (__temp132 == NULL ? NULL : g_object_ref (__temp132))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp133);
}


static void vala_semantic_analyzer_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp135 = NULL;
	ValaSymbol* __temp134 = NULL;
	self->priv->current_symbol = (__temp135 = (__temp134 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp134 == NULL ? NULL : g_object_ref (__temp134))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp135);
}


static void vala_semantic_analyzer_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp137 = NULL;
	ValaSymbol* __temp136 = NULL;
	self->priv->current_symbol = (__temp137 = (__temp136 = vala_code_node_get_symbol (VALA_CODE_NODE (d)), (__temp136 == NULL ? NULL : g_object_ref (__temp136))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp137);
}


static void vala_semantic_analyzer_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp139 = NULL;
	ValaSymbol* __temp138 = NULL;
	self->priv->current_symbol = (__temp139 = (__temp138 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp138 == NULL ? NULL : g_object_ref (__temp138))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp139);
}


static void vala_semantic_analyzer_real_visit_named_argument (ValaCodeVisitor* base, ValaNamedArgument* n)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMED_ARGUMENT (n));
}


static void vala_semantic_analyzer_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	ValaSymbol* __temp141 = NULL;
	ValaSymbol* __temp140 = NULL;
	self->priv->current_symbol = (__temp141 = (__temp140 = vala_code_node_get_symbol (VALA_CODE_NODE (b)), (__temp140 == NULL ? NULL : g_object_ref (__temp140))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp141);
}


static void vala_semantic_analyzer_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	{
		GList* __temp142 = NULL;
		__temp142 = vala_block_get_local_variables (b);
		GList* decl_it;
		for (decl_it = __temp142; decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				(vala_symbol_set_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl)), FALSE), vala_symbol_get_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl))));
			}
		}
		(__temp142 == NULL ? NULL : (__temp142 = (g_list_free (__temp142), NULL)));
	}
	ValaSymbol* __temp144 = NULL;
	ValaSymbol* __temp143 = NULL;
	self->priv->current_symbol = (__temp144 = (__temp143 = vala_symbol_get_parent_symbol (self->priv->current_symbol), (__temp143 == NULL ? NULL : g_object_ref (__temp143))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp144);
}


static void vala_semantic_analyzer_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_VARIABLE_DECLARATOR (decl));
	if (vala_variable_declarator_get_type_reference (decl) == NULL) {
		/* var type */
		if (vala_variable_declarator_get_initializer (decl) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "var declaration not allowed without initializer");
			return;
		}
		if (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "var declaration not allowed with non-typed initializer");
			return;
		}
		ValaTypeReference* __temp145 = NULL;
		(vala_variable_declarator_set_type_reference (decl, (__temp145 = vala_type_reference_copy (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl))))), vala_variable_declarator_get_type_reference (decl));
		(__temp145 == NULL ? NULL : (__temp145 = (g_object_unref (__temp145), NULL)));
		(vala_type_reference_set_takes_ownership (vala_variable_declarator_get_type_reference (decl), (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)) == NULL || vala_data_type_is_reference_type (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))))), vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl)));
		(vala_type_reference_set_transfers_ownership (vala_variable_declarator_get_type_reference (decl), FALSE), vala_type_reference_get_transfers_ownership (vala_variable_declarator_get_type_reference (decl)));
	}
	if (vala_variable_declarator_get_initializer (decl) != NULL) {
		if (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)) == NULL) {
			if (!(VALA_IS_MEMBER_ACCESS (vala_variable_declarator_get_initializer (decl)))) {
				(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "expression type not allowed as initializer");
				return;
			}
			if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_variable_declarator_get_initializer (decl)))) && VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
				ValaMethod* __temp146 = NULL;
				ValaMethod* m = (__temp146 = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_variable_declarator_get_initializer (decl)))), (__temp146 == NULL ? NULL : g_object_ref (__temp146)));
				ValaCallback* __temp147 = NULL;
				ValaCallback* cb = (__temp147 = VALA_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))), (__temp147 == NULL ? NULL : g_object_ref (__temp147)));
				/* check whether method matches callback type */
				if (!vala_callback_matches_method (cb, m)) {
					(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
					char* __temp150 = NULL;
					char* __temp149 = NULL;
					char* __temp148 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), (__temp150 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp148 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp149 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
					(__temp150 == NULL ? NULL : (__temp150 = (g_free (__temp150), NULL)));
					(__temp149 == NULL ? NULL : (__temp149 = (g_free (__temp149), NULL)));
					(__temp148 == NULL ? NULL : (__temp148 = (g_free (__temp148), NULL)));
					(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
					(cb == NULL ? NULL : (cb = (g_object_unref (cb), NULL)));
					return;
				}
				(vala_expression_set_static_type (vala_variable_declarator_get_initializer (decl), vala_variable_declarator_get_type_reference (decl)), vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)));
				(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
				(cb == NULL ? NULL : (cb = (g_object_unref (cb), NULL)));
			} else {
				(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "expression type not allowed as initializer");
				return;
			}
		}
		if (vala_semantic_analyzer_get_memory_management (self)) {
			if (vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)))) {
				/* rhs transfers ownership of the expression */
				if (!vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl))) {
					/* lhs doesn't own the value
					 * promote lhs type */
					(vala_type_reference_set_takes_ownership (vala_variable_declarator_get_type_reference (decl), TRUE), vala_type_reference_get_takes_ownership (vala_variable_declarator_get_type_reference (decl)));
				}
			}
		}
	}
	if (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)) != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
	ValaSymbol* __temp151 = NULL;
	(vala_code_node_set_symbol (VALA_CODE_NODE (decl), (__temp151 = vala_symbol_new (VALA_CODE_NODE (decl)))), vala_code_node_get_symbol (VALA_CODE_NODE (decl)));
	(__temp151 == NULL ? NULL : (__temp151 = (g_object_unref (__temp151), NULL)));
	vala_symbol_add (self->priv->current_symbol, vala_variable_declarator_get_name (decl), vala_code_node_get_symbol (VALA_CODE_NODE (decl)));
	ValaBlock* __temp152 = NULL;
	ValaBlock* block = (__temp152 = VALA_BLOCK (vala_symbol_get_node (self->priv->current_symbol)), (__temp152 == NULL ? NULL : g_object_ref (__temp152)));
	vala_block_add_local_variable (block, decl);
	(vala_symbol_set_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl)), TRUE), vala_symbol_get_active (vala_code_node_get_symbol (VALA_CODE_NODE (decl))));
	(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
}


/**
 * Visit operation called for initializer lists
 *
 * @param list an initializer list
 */
static void vala_semantic_analyzer_real_visit_begin_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INITIALIZER_LIST (list));
	if (vala_expression_get_expected_type (VALA_EXPRESSION (list)) != NULL && VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (list))))) {
		/* initializer is used as array initializer */
		ValaArray* __temp153 = NULL;
		ValaArray* edt = (__temp153 = VALA_ARRAY (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (list)))), (__temp153 == NULL ? NULL : g_object_ref (__temp153)));
		GList* inits = vala_initializer_list_get_initializers (list);
		gint rank = vala_array_get_rank ((VALA_ARRAY (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (list))))));
		ValaTypeReference* child_type = vala_type_reference_copy (vala_expression_get_expected_type (VALA_EXPRESSION (list)));
		if (rank > 1) {
			ValaArray* __temp154 = NULL;
			(vala_type_reference_set_data_type (child_type, VALA_DATA_TYPE ((__temp154 = vala_data_type_get_array (vala_array_get_element_type (edt), rank - 1)))), vala_type_reference_get_data_type (child_type));
			(__temp154 == NULL ? NULL : (__temp154 = (g_object_unref (__temp154), NULL)));
		} else {
			(vala_type_reference_set_data_type (child_type, vala_array_get_element_type (edt)), vala_type_reference_get_data_type (child_type));
		}
		{
			GList* __temp156 = NULL;
			__temp156 = inits;
			GList* e_it;
			for (e_it = __temp156; e_it != NULL; e_it = e_it->next) {
				ValaExpression* e = e_it->data;
				{
					ValaTypeReference* __temp155 = NULL;
					(vala_expression_set_expected_type (e, (__temp155 = vala_type_reference_copy (child_type))), vala_expression_get_expected_type (e));
					(__temp155 == NULL ? NULL : (__temp155 = (g_object_unref (__temp155), NULL)));
				}
			}
		}
		(edt == NULL ? NULL : (edt = (g_object_unref (edt), NULL)));
		(inits == NULL ? NULL : (inits = (g_list_free (inits), NULL)));
		(child_type == NULL ? NULL : (child_type = (g_object_unref (child_type), NULL)));
	}
}


/**
 * Visit operation called for initializer lists
 *
 * @param list an initializer list
 */
static void vala_semantic_analyzer_real_visit_end_initializer_list (ValaCodeVisitor* base, ValaInitializerList* list)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INITIALIZER_LIST (list));
	if (vala_expression_get_expected_type (VALA_EXPRESSION (list)) != NULL && VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (list))))) {
		ValaArray* __temp157 = NULL;
		ValaArray* edt = (__temp157 = VALA_ARRAY (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (list)))), (__temp157 == NULL ? NULL : g_object_ref (__temp157)));
		GList* inits = vala_initializer_list_get_initializers (list);
		gint rank = vala_array_get_rank (edt);
		ValaTypeReference* child_type = vala_type_reference_copy (vala_expression_get_expected_type (VALA_EXPRESSION (list)));
		gboolean error = FALSE;
		if (rank > 1) {
			ValaArray* __temp158 = NULL;
			(vala_type_reference_set_data_type (child_type, VALA_DATA_TYPE ((__temp158 = vala_data_type_get_array (vala_array_get_element_type (edt), rank - 1)))), vala_type_reference_get_data_type (child_type));
			(__temp158 == NULL ? NULL : (__temp158 = (g_object_unref (__temp158), NULL)));
			{
				GList* __temp160 = NULL;
				__temp160 = inits;
				GList* e_it;
				for (e_it = __temp160; e_it != NULL; e_it = e_it->next) {
					ValaExpression* e = e_it->data;
					{
						if (vala_expression_get_static_type (e) == NULL) {
							error = TRUE;
							continue;
						}
						if (!(VALA_IS_INITIALIZER_LIST (e))) {
							error = TRUE;
							(vala_code_node_set_error (VALA_CODE_NODE (e), TRUE), vala_code_node_get_error (VALA_CODE_NODE (e)));
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (e)), "Initializer list expected");
							continue;
						}
						if (!vala_type_reference_equals (vala_expression_get_static_type (e), child_type)) {
							error = TRUE;
							(vala_code_node_set_error (VALA_CODE_NODE (e), TRUE), vala_code_node_get_error (VALA_CODE_NODE (e)));
							char* __temp159 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (e)), (__temp159 = g_strdup_printf ("Expected initializer list of type `%s' but got `%s'", vala_data_type_get_name (vala_type_reference_get_data_type (child_type)), vala_data_type_get_name (vala_type_reference_get_data_type (vala_expression_get_static_type (e))))));
							(__temp159 == NULL ? NULL : (__temp159 = (g_free (__temp159), NULL)));
						}
					}
				}
			}
		} else {
			(vala_type_reference_set_data_type (child_type, vala_array_get_element_type (edt)), vala_type_reference_get_data_type (child_type));
			{
				GList* __temp162 = NULL;
				__temp162 = inits;
				GList* e_it;
				for (e_it = __temp162; e_it != NULL; e_it = e_it->next) {
					ValaExpression* e = e_it->data;
					{
						if (vala_expression_get_static_type (e) == NULL) {
							error = TRUE;
							continue;
						}
						if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (e), child_type)) {
							error = TRUE;
							(vala_code_node_set_error (VALA_CODE_NODE (e), TRUE), vala_code_node_get_error (VALA_CODE_NODE (e)));
							char* __temp161 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (e)), (__temp161 = g_strdup_printf ("Expected initializer of type `%s' but got `%s'", vala_data_type_get_name (vala_type_reference_get_data_type (child_type)), vala_data_type_get_name (vala_type_reference_get_data_type (vala_expression_get_static_type (e))))));
							(__temp161 == NULL ? NULL : (__temp161 = (g_free (__temp161), NULL)));
						}
					}
				}
			}
		}
		if (!error) {
			/* everything seems to be correct */
			(vala_expression_set_static_type (VALA_EXPRESSION (list), vala_expression_get_expected_type (VALA_EXPRESSION (list))), vala_expression_get_static_type (VALA_EXPRESSION (list)));
		}
		(edt == NULL ? NULL : (edt = (g_object_unref (edt), NULL)));
		(inits == NULL ? NULL : (inits = (g_list_free (inits), NULL)));
		(child_type == NULL ? NULL : (child_type = (g_object_unref (child_type), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_EXPRESSION_STATEMENT (stmt));
	if (vala_expression_get_static_type (vala_expression_statement_get_expression (stmt)) != NULL && vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_expression_statement_get_expression (stmt)))) {
		vala_report_warning (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Short-living reference");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_IF_STATEMENT (stmt));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_if_statement_get_condition (stmt)))) {
		/* if there was an error in the condition, skip this check */
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		return;
	}
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_if_statement_get_condition (stmt))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_if_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_WHILE_STATEMENT (stmt));
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_while_statement_get_condition (stmt))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_while_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FOR_STATEMENT (stmt));
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_for_statement_get_condition (stmt))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_for_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_begin_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (stmt));
	if (vala_type_reference_get_data_type (vala_foreach_statement_get_type_reference (stmt)) != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_foreach_statement_get_type_reference (stmt)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
	ValaVariableDeclarator* __temp163 = NULL;
	(vala_foreach_statement_set_variable_declarator (stmt, (__temp163 = vala_variable_declarator_new (vala_foreach_statement_get_variable_name (stmt), NULL, NULL))), vala_foreach_statement_get_variable_declarator (stmt));
	(__temp163 == NULL ? NULL : (__temp163 = (g_object_unref (__temp163), NULL)));
	(vala_variable_declarator_set_type_reference (vala_foreach_statement_get_variable_declarator (stmt), vala_foreach_statement_get_type_reference (stmt)), vala_variable_declarator_get_type_reference (vala_foreach_statement_get_variable_declarator (stmt)));
	ValaSymbol* __temp164 = NULL;
	(vala_code_node_set_symbol (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt)), (__temp164 = vala_symbol_new (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt))))), vala_code_node_get_symbol (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt))));
	(__temp164 == NULL ? NULL : (__temp164 = (g_object_unref (__temp164), NULL)));
	vala_symbol_add (vala_code_node_get_symbol (VALA_CODE_NODE (vala_foreach_statement_get_body (stmt))), vala_foreach_statement_get_variable_name (stmt), vala_code_node_get_symbol (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt))));
}


static void vala_semantic_analyzer_real_visit_end_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (stmt));
	if (self->priv->current_return_type == NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return not allowed in this context");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) == NULL && vala_type_reference_get_data_type (self->priv->current_return_type) != NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return without value in non-void function");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && vala_type_reference_get_data_type (self->priv->current_return_type) == NULL && vala_type_reference_get_type_parameter (self->priv->current_return_type) == NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return with value in void function");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt)), self->priv->current_return_type)) {
		char* __temp167 = NULL;
		char* __temp166 = NULL;
		char* __temp165 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), (__temp167 = g_strdup_printf ("Return: Cannot convert from `%s' to `%s'", (__temp165 = vala_type_reference_to_string (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt)))), (__temp166 = vala_type_reference_to_string (self->priv->current_return_type)))));
		(__temp167 == NULL ? NULL : (__temp167 = (g_free (__temp167), NULL)));
		(__temp166 == NULL ? NULL : (__temp166 = (g_free (__temp166), NULL)));
		(__temp165 == NULL ? NULL : (__temp165 = (g_free (__temp165), NULL)));
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt))) && !vala_type_reference_get_transfers_ownership (self->priv->current_return_type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return value transfers ownership but method return type hasn't been declared to transfer ownership");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt)) != NULL && VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_return_statement_get_return_expression (stmt)))) && vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt))) && !vala_type_reference_get_transfers_ownership (self->priv->current_return_type)) {
		vala_report_warning (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Local variable with strong reference used as return value and method return type hasn't been declared to transfer ownership");
	}
}


/**
 * Visit operation called for lock statements.
 *
 * @param stmt a lock statement
 */
static void vala_semantic_analyzer_real_visit_lock_statement (ValaCodeVisitor* base, ValaLockStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_LOCK_STATEMENT (stmt));
	/* resource must be a member access and denote a Lockable */
	if (!(VALA_IS_MEMBER_ACCESS (vala_lock_statement_get_resource (stmt)) && VALA_IS_LOCKABLE (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt)))))) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		(vala_code_node_set_error (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt)), TRUE), vala_code_node_get_error (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt))));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt))), "Expression is either not a member access or does not denote a lockable member");
		return;
	}
	/* parent symbol must be the current class */
	if (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt)))) != VALA_CODE_NODE (self->priv->current_class)) {
		(vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE), vala_code_node_get_error (VALA_CODE_NODE (stmt)));
		(vala_code_node_set_error (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt)), TRUE), vala_code_node_get_error (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt))));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_lock_statement_get_resource (stmt))), "Only members of the current class are lockable");
	}
	vala_lockable_set_lock_used ((VALA_LOCKABLE (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_lock_statement_get_resource (stmt))))), TRUE);
}


static void vala_semantic_analyzer_real_visit_begin_array_creation_expression (ValaCodeVisitor* base, ValaArrayCreationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ARRAY_CREATION_EXPRESSION (expr));
	if (vala_array_creation_expression_get_initializer_list (expr) != NULL) {
		ValaTypeReference* __temp168 = NULL;
		(vala_expression_set_expected_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr)), (__temp168 = vala_type_reference_copy (vala_array_creation_expression_get_element_type (expr)))), vala_expression_get_expected_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr))));
		(__temp168 == NULL ? NULL : (__temp168 = (g_object_unref (__temp168), NULL)));
		ValaArray* __temp169 = NULL;
		(vala_type_reference_set_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr))), VALA_DATA_TYPE ((__temp169 = vala_data_type_get_array (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr)))), vala_array_creation_expression_get_rank (expr))))), vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr)))));
		(__temp169 == NULL ? NULL : (__temp169 = (g_object_unref (__temp169), NULL)));
	}
}


/* FIXME: add element type to type_argument
*
 * Visit operations called for array creation expresions.
 *
 * @param expr an array creation expression
 */
static void vala_semantic_analyzer_real_visit_end_array_creation_expression (ValaCodeVisitor* base, ValaArrayCreationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ARRAY_CREATION_EXPRESSION (expr));
	gint i;
	memset (&i, 0, sizeof (gint));
	GList* size = vala_array_creation_expression_get_sizes (expr);
	/* check for errors in the size list */
	if (size != NULL) {
		{
			GList* __temp170 = NULL;
			__temp170 = size;
			GList* e_it;
			for (e_it = __temp170; e_it != NULL; e_it = e_it->next) {
				ValaExpression* e = e_it->data;
				{
					if (vala_expression_get_static_type (e) == NULL) {
						(size == NULL ? NULL : (size = (g_list_free (size), NULL)));
						return;
					} else {
						/* return on previous error */
						if (vala_type_reference_get_data_type (vala_expression_get_static_type (e)) != vala_type_reference_get_data_type (self->priv->int_type)) {
							(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (e)), "Expected expression of type ´int'");
						}
					}
				}
			}
		}
		if (vala_code_node_get_error (VALA_CODE_NODE (expr))) {
			(size == NULL ? NULL : (size = (g_list_free (size), NULL)));
			return;
		}
	}
	/* check for wrong elements inside the initializer */
	if (vala_array_creation_expression_get_initializer_list (expr) != NULL && vala_expression_get_static_type (VALA_EXPRESSION (vala_array_creation_expression_get_initializer_list (expr))) == NULL) {
		(size == NULL ? NULL : (size = (g_list_free (size), NULL)));
		return;
	}
	/* try to construct the type of the array */
	if (vala_array_creation_expression_get_element_type (expr) == NULL) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Cannot determine the element type of the created array");
		(size == NULL ? NULL : (size = (g_list_free (size), NULL)));
		return;
	}
	ValaTypeReference* __temp171 = NULL;
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp171 = vala_type_reference_copy (vala_array_creation_expression_get_element_type (expr)))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	(__temp171 == NULL ? NULL : (__temp171 = (g_object_unref (__temp171), NULL)));
	if (vala_type_reference_get_data_type (vala_array_creation_expression_get_element_type (expr)) != NULL) {
		ValaArray* __temp172 = NULL;
		(vala_type_reference_set_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)), VALA_DATA_TYPE ((__temp172 = vala_data_type_get_array (vala_type_reference_get_data_type (vala_array_creation_expression_get_element_type (expr)), vala_array_creation_expression_get_rank (expr))))), vala_type_reference_get_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
		(__temp172 == NULL ? NULL : (__temp172 = (g_object_unref (__temp172), NULL)));
	} else {
		ValaArray* __temp173 = NULL;
		(vala_type_reference_set_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)), VALA_DATA_TYPE ((__temp173 = vala_type_parameter_get_array (vala_type_reference_get_type_parameter (vala_array_creation_expression_get_element_type (expr)), vala_array_creation_expression_get_rank (expr))))), vala_type_reference_get_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
		(__temp173 == NULL ? NULL : (__temp173 = (g_object_unref (__temp173), NULL)));
	}
	(vala_type_reference_set_transfers_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE), vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	(vala_type_reference_set_takes_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE), vala_type_reference_get_takes_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	vala_type_reference_add_type_argument (vala_expression_get_static_type (VALA_EXPRESSION (expr)), vala_array_creation_expression_get_element_type (expr));
	(size == NULL ? NULL : (size = (g_list_free (size), NULL)));
}


static void vala_semantic_analyzer_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BOOLEAN_LITERAL (expr));
	(vala_literal_set_static_type (VALA_LITERAL (expr), self->priv->bool_type), vala_literal_get_static_type (VALA_LITERAL (expr)));
}


static void vala_semantic_analyzer_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CHARACTER_LITERAL (expr));
	ValaTypeReference* __temp174 = NULL;
	(vala_literal_set_static_type (VALA_LITERAL (expr), (__temp174 = vala_type_reference_new ())), vala_literal_get_static_type (VALA_LITERAL (expr)));
	(__temp174 == NULL ? NULL : (__temp174 = (g_object_unref (__temp174), NULL)));
	ValaSymbol* __temp175 = NULL;
	(vala_type_reference_set_data_type (vala_literal_get_static_type (VALA_LITERAL (expr)), VALA_DATA_TYPE (vala_symbol_get_node ((__temp175 = vala_symbol_lookup (self->priv->root_symbol, "char"))))), vala_type_reference_get_data_type (vala_literal_get_static_type (VALA_LITERAL (expr))));
	(__temp175 == NULL ? NULL : (__temp175 = (g_object_unref (__temp175), NULL)));
}


static void vala_semantic_analyzer_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INTEGER_LITERAL (expr));
	ValaTypeReference* __temp176 = NULL;
	(vala_literal_set_static_type (VALA_LITERAL (expr), (__temp176 = vala_type_reference_new ())), vala_literal_get_static_type (VALA_LITERAL (expr)));
	(__temp176 == NULL ? NULL : (__temp176 = (g_object_unref (__temp176), NULL)));
	ValaSymbol* __temp178 = NULL;
	char* __temp177 = NULL;
	(vala_type_reference_set_data_type (vala_literal_get_static_type (VALA_LITERAL (expr)), VALA_DATA_TYPE (vala_symbol_get_node ((__temp178 = vala_symbol_lookup (self->priv->root_symbol, (__temp177 = vala_integer_literal_get_type_name (expr))))))), vala_type_reference_get_data_type (vala_literal_get_static_type (VALA_LITERAL (expr))));
	(__temp178 == NULL ? NULL : (__temp178 = (g_object_unref (__temp178), NULL)));
	(__temp177 == NULL ? NULL : (__temp177 = (g_free (__temp177), NULL)));
}


static void vala_semantic_analyzer_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_REAL_LITERAL (expr));
	ValaTypeReference* __temp179 = NULL;
	(vala_literal_set_static_type (VALA_LITERAL (expr), (__temp179 = vala_type_reference_new ())), vala_literal_get_static_type (VALA_LITERAL (expr)));
	(__temp179 == NULL ? NULL : (__temp179 = (g_object_unref (__temp179), NULL)));
	ValaSymbol* __temp181 = NULL;
	char* __temp180 = NULL;
	(vala_type_reference_set_data_type (vala_literal_get_static_type (VALA_LITERAL (expr)), VALA_DATA_TYPE (vala_symbol_get_node ((__temp181 = vala_symbol_lookup (self->priv->root_symbol, (__temp180 = vala_real_literal_get_type_name (expr))))))), vala_type_reference_get_data_type (vala_literal_get_static_type (VALA_LITERAL (expr))));
	(__temp181 == NULL ? NULL : (__temp181 = (g_object_unref (__temp181), NULL)));
	(__temp180 == NULL ? NULL : (__temp180 = (g_free (__temp180), NULL)));
}


static void vala_semantic_analyzer_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRING_LITERAL (expr));
	ValaTypeReference* __temp182 = NULL;
	(vala_literal_set_static_type (VALA_LITERAL (expr), (__temp182 = vala_type_reference_copy (self->priv->string_type))), vala_literal_get_static_type (VALA_LITERAL (expr)));
	(__temp182 == NULL ? NULL : (__temp182 = (g_object_unref (__temp182), NULL)));
	(vala_type_reference_set_non_null (vala_literal_get_static_type (VALA_LITERAL (expr)), TRUE), vala_type_reference_get_non_null (vala_literal_get_static_type (VALA_LITERAL (expr))));
}


static void vala_semantic_analyzer_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NULL_LITERAL (expr));
	/* empty TypeReference represents null */
	ValaTypeReference* __temp183 = NULL;
	(vala_literal_set_static_type (VALA_LITERAL (expr), (__temp183 = vala_type_reference_new ())), vala_literal_get_static_type (VALA_LITERAL (expr)));
	(__temp183 == NULL ? NULL : (__temp183 = (g_object_unref (__temp183), NULL)));
}


static void vala_semantic_analyzer_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_LITERAL_EXPRESSION (expr));
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_literal_get_static_type (vala_literal_expression_get_literal (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static ValaTypeReference* vala_semantic_analyzer_get_static_type_for_node (ValaSemanticAnalyzer* self, ValaCodeNode* node)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_CODE_NODE (node), NULL);
	if (VALA_IS_FIELD (node)) {
		ValaField* __temp184 = NULL;
		ValaField* f = (__temp184 = VALA_FIELD (node), (__temp184 == NULL ? NULL : g_object_ref (__temp184)));
		ValaTypeReference* __temp185 = NULL;
		return (__temp185 = g_object_ref (vala_field_get_type_reference (f)), (f == NULL ? NULL : (f = (g_object_unref (f), NULL))), __temp185);
		(f == NULL ? NULL : (f = (g_object_unref (f), NULL)));
	} else {
		if (VALA_IS_CONSTANT (node)) {
			ValaConstant* __temp186 = NULL;
			ValaConstant* c = (__temp186 = VALA_CONSTANT (node), (__temp186 == NULL ? NULL : g_object_ref (__temp186)));
			ValaTypeReference* __temp187 = NULL;
			return (__temp187 = g_object_ref (vala_constant_get_type_reference (c)), (c == NULL ? NULL : (c = (g_object_unref (c), NULL))), __temp187);
			(c == NULL ? NULL : (c = (g_object_unref (c), NULL)));
		} else {
			if (VALA_IS_PROPERTY (node)) {
				ValaProperty* __temp188 = NULL;
				ValaProperty* prop = (__temp188 = VALA_PROPERTY (node), (__temp188 == NULL ? NULL : g_object_ref (__temp188)));
				ValaTypeReference* type = vala_type_reference_copy (vala_property_get_type_reference (prop));
				(vala_type_reference_set_takes_ownership (type, FALSE), vala_type_reference_get_takes_ownership (type));
				ValaTypeReference* __temp189 = NULL;
				return (__temp189 = type, (prop == NULL ? NULL : (prop = (g_object_unref (prop), NULL))), __temp189);
				(prop == NULL ? NULL : (prop = (g_object_unref (prop), NULL)));
				(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
			} else {
				if (VALA_IS_FORMAL_PARAMETER (node)) {
					ValaFormalParameter* __temp190 = NULL;
					ValaFormalParameter* p = (__temp190 = VALA_FORMAL_PARAMETER (node), (__temp190 == NULL ? NULL : g_object_ref (__temp190)));
					ValaTypeReference* __temp191 = NULL;
					ValaTypeReference* __temp192 = NULL;
					return (__temp192 = (__temp191 = vala_formal_parameter_get_type_reference (p), (__temp191 == NULL ? NULL : g_object_ref (__temp191))), (p == NULL ? NULL : (p = (g_object_unref (p), NULL))), __temp192);
					(p == NULL ? NULL : (p = (g_object_unref (p), NULL)));
				} else {
					if (VALA_IS_TYPE_REFERENCE (node)) {
						ValaTypeReference* __temp193 = NULL;
						return (__temp193 = VALA_TYPE_REFERENCE (node), (__temp193 == NULL ? NULL : g_object_ref (__temp193)));
					} else {
						if (VALA_IS_VARIABLE_DECLARATOR (node)) {
							ValaVariableDeclarator* __temp195 = NULL;
							ValaVariableDeclarator* decl = (__temp195 = VALA_VARIABLE_DECLARATOR (node), (__temp195 == NULL ? NULL : g_object_ref (__temp195)));
							ValaTypeReference* __temp196 = NULL;
							ValaTypeReference* __temp197 = NULL;
							return (__temp197 = (__temp196 = vala_variable_declarator_get_type_reference (decl), (__temp196 == NULL ? NULL : g_object_ref (__temp196))), (decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL))), __temp197);
							(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
						} else {
							if (VALA_IS_ENUM_VALUE (node)) {
								ValaTypeReference* type = vala_type_reference_new ();
								(vala_type_reference_set_data_type (type, VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (node))))), vala_type_reference_get_data_type (type));
								return type;
								(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
							}
						}
					}
				}
			}
		}
	}
	return VALA_TYPE_REFERENCE (NULL);
}


ValaSymbol* vala_semantic_analyzer_symbol_lookup_inherited (ValaSymbol* sym, char* name)
{
	g_return_val_if_fail (VALA_IS_SYMBOL (sym), NULL);
	g_return_val_if_fail (name != NULL, NULL);
	ValaSymbol* result = vala_symbol_lookup (sym, name);
	if (result != NULL) {
		return result;
	}
	if (VALA_IS_CLASS (vala_symbol_get_node (sym))) {
		ValaClass* __temp201 = NULL;
		ValaClass* cl = (__temp201 = VALA_CLASS (vala_symbol_get_node (sym)), (__temp201 == NULL ? NULL : g_object_ref (__temp201)));
		{
			GList* __temp204 = NULL;
			__temp204 = vala_class_get_base_types (cl);
			GList* base_type_it;
			for (base_type_it = __temp204; base_type_it != NULL; base_type_it = base_type_it->next) {
				ValaTypeReference* base_type = base_type_it->data;
				{
					ValaSymbol* __temp202 = NULL;
					result = (__temp202 = vala_semantic_analyzer_symbol_lookup_inherited (vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (base_type))), name), (result == NULL ? NULL : (result = (g_object_unref (result), NULL))), __temp202);
					if (result != NULL) {
						ValaSymbol* __temp203 = NULL;
						return (__temp203 = result, (cl == NULL ? NULL : (cl = (g_object_unref (cl), NULL))), __temp203);
					}
				}
			}
			(__temp204 == NULL ? NULL : (__temp204 = (g_list_free (__temp204), NULL)));
		}
		(cl == NULL ? NULL : (cl = (g_object_unref (cl), NULL)));
	} else {
		if (VALA_IS_STRUCT (vala_symbol_get_node (sym))) {
			ValaStruct* __temp205 = NULL;
			ValaStruct* st = (__temp205 = VALA_STRUCT (vala_symbol_get_node (sym)), (__temp205 == NULL ? NULL : g_object_ref (__temp205)));
			{
				GList* __temp208 = NULL;
				__temp208 = vala_struct_get_base_types (st);
				GList* base_type_it;
				for (base_type_it = __temp208; base_type_it != NULL; base_type_it = base_type_it->next) {
					ValaTypeReference* base_type = base_type_it->data;
					{
						ValaSymbol* __temp206 = NULL;
						result = (__temp206 = vala_semantic_analyzer_symbol_lookup_inherited (vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (base_type))), name), (result == NULL ? NULL : (result = (g_object_unref (result), NULL))), __temp206);
						if (result != NULL) {
							ValaSymbol* __temp207 = NULL;
							return (__temp207 = result, (st == NULL ? NULL : (st = (g_object_unref (st), NULL))), __temp207);
						}
					}
				}
				(__temp208 == NULL ? NULL : (__temp208 = (g_list_free (__temp208), NULL)));
			}
			(st == NULL ? NULL : (st = (g_object_unref (st), NULL)));
		} else {
			if (VALA_IS_INTERFACE (vala_symbol_get_node (sym))) {
				ValaInterface* __temp209 = NULL;
				ValaInterface* iface = (__temp209 = VALA_INTERFACE (vala_symbol_get_node (sym)), (__temp209 == NULL ? NULL : g_object_ref (__temp209)));
				{
					GList* __temp212 = NULL;
					__temp212 = vala_interface_get_prerequisites (iface);
					GList* prerequisite_it;
					for (prerequisite_it = __temp212; prerequisite_it != NULL; prerequisite_it = prerequisite_it->next) {
						ValaTypeReference* prerequisite = prerequisite_it->data;
						{
							ValaSymbol* __temp210 = NULL;
							result = (__temp210 = vala_semantic_analyzer_symbol_lookup_inherited (vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (prerequisite))), name), (result == NULL ? NULL : (result = (g_object_unref (result), NULL))), __temp210);
							if (result != NULL) {
								ValaSymbol* __temp211 = NULL;
								return (__temp211 = result, (iface == NULL ? NULL : (iface = (g_object_unref (iface), NULL))), __temp211);
							}
						}
					}
					(__temp212 == NULL ? NULL : (__temp212 = (g_list_free (__temp212), NULL)));
				}
				(iface == NULL ? NULL : (iface = (g_object_unref (iface), NULL)));
			}
		}
	}
	gpointer __temp213;
	return VALA_SYMBOL ((__temp213 = NULL, (result == NULL ? NULL : (result = (g_object_unref (result), NULL))), __temp213));
	(result == NULL ? NULL : (result = (g_object_unref (result), NULL)));
}


static void vala_semantic_analyzer_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PARENTHESIZED_EXPRESSION (expr));
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_parenthesized_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static ValaDataType* vala_semantic_analyzer_find_parent_type (ValaSemanticAnalyzer* self, ValaSymbol* sym)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (sym == NULL || VALA_IS_SYMBOL (sym), NULL);
	while (sym != NULL) {
		if (VALA_IS_DATA_TYPE (vala_symbol_get_node (sym))) {
			ValaDataType* __temp214 = NULL;
			return (__temp214 = VALA_DATA_TYPE (vala_symbol_get_node (sym)), (__temp214 == NULL ? NULL : g_object_ref (__temp214)));
		}
		sym = vala_symbol_get_parent_symbol (sym);
	}
	return VALA_DATA_TYPE (NULL);
}


static void vala_semantic_analyzer_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (expr));
	ValaSymbol* base_symbol = NULL;
	if (vala_member_access_get_inner (expr) == NULL) {
		ValaSymbol* __temp218 = NULL;
		ValaSymbol* __temp217 = NULL;
		base_symbol = (__temp218 = (__temp217 = self->priv->current_symbol, (__temp217 == NULL ? NULL : g_object_ref (__temp217))), (base_symbol == NULL ? NULL : (base_symbol = (g_object_unref (base_symbol), NULL))), __temp218);
		ValaSymbol* __temp219 = NULL;
		ValaSymbol* sym = (__temp219 = self->priv->current_symbol, (__temp219 == NULL ? NULL : g_object_ref (__temp219)));
		while (sym != NULL && vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
			ValaSymbol* __temp220 = NULL;
			(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), (__temp220 = vala_semantic_analyzer_symbol_lookup_inherited (sym, vala_member_access_get_member_name (expr)))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
			(__temp220 == NULL ? NULL : (__temp220 = (g_object_unref (__temp220), NULL)));
			ValaSymbol* __temp222 = NULL;
			ValaSymbol* __temp221 = NULL;
			sym = (__temp222 = (__temp221 = vala_symbol_get_parent_symbol (sym), (__temp221 == NULL ? NULL : g_object_ref (__temp221))), (sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL))), __temp222);
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
			{
				GList* __temp226 = NULL;
				__temp226 = self->priv->current_using_directives;
				GList* ns_it;
				for (ns_it = __temp226; ns_it != NULL; ns_it = ns_it->next) {
					ValaNamespaceReference* ns = ns_it->data;
					{
						ValaSymbol* local_sym = vala_symbol_lookup (vala_namespace_reference_get_namespace_symbol (ns), vala_member_access_get_member_name (expr));
						if (local_sym != NULL) {
							if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) != NULL) {
								(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
								char* __temp225 = NULL;
								char* __temp224 = NULL;
								char* __temp223 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp225 = g_strdup_printf ("`%s' is an ambiguous reference between `%s' and `%s'", vala_member_access_get_member_name (expr), (__temp223 = vala_symbol_get_full_name (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)))), (__temp224 = vala_symbol_get_full_name (local_sym)))));
								(__temp225 == NULL ? NULL : (__temp225 = (g_free (__temp225), NULL)));
								(__temp224 == NULL ? NULL : (__temp224 = (g_free (__temp224), NULL)));
								(__temp223 == NULL ? NULL : (__temp223 = (g_free (__temp223), NULL)));
								(local_sym == NULL ? NULL : (local_sym = (g_object_unref (local_sym), NULL)));
								(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
								(base_symbol == NULL ? NULL : (base_symbol = (g_object_unref (base_symbol), NULL)));
								return;
							}
							(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), local_sym), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
						}
						(local_sym == NULL ? NULL : (local_sym = (g_object_unref (local_sym), NULL)));
					}
				}
			}
		}
		(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
	} else {
		if (vala_code_node_get_error (VALA_CODE_NODE (vala_member_access_get_inner (expr)))) {
			/* if there was an error in the inner expression, skip this check */
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			(base_symbol == NULL ? NULL : (base_symbol = (g_object_unref (base_symbol), NULL)));
			return;
		}
		if (VALA_IS_MEMBER_ACCESS (vala_member_access_get_inner (expr)) || VALA_IS_BASE_ACCESS (vala_member_access_get_inner (expr))) {
			ValaSymbol* __temp228 = NULL;
			ValaSymbol* __temp227 = NULL;
			base_symbol = (__temp228 = (__temp227 = vala_expression_get_symbol_reference (vala_member_access_get_inner (expr)), (__temp227 == NULL ? NULL : g_object_ref (__temp227))), (base_symbol == NULL ? NULL : (base_symbol = (g_object_unref (base_symbol), NULL))), __temp228);
			if (VALA_IS_NAMESPACE (vala_symbol_get_node (base_symbol)) || VALA_IS_DATA_TYPE (vala_symbol_get_node (base_symbol))) {
				ValaSymbol* __temp229 = NULL;
				(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), (__temp229 = vala_symbol_lookup (base_symbol, vala_member_access_get_member_name (expr)))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
				(__temp229 == NULL ? NULL : (__temp229 = (g_object_unref (__temp229), NULL)));
			}
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && vala_expression_get_static_type (vala_member_access_get_inner (expr)) != NULL) {
			ValaSymbol* __temp231 = NULL;
			ValaSymbol* __temp230 = NULL;
			base_symbol = (__temp231 = (__temp230 = vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_member_access_get_inner (expr))))), (__temp230 == NULL ? NULL : g_object_ref (__temp230))), (base_symbol == NULL ? NULL : (base_symbol = (g_object_unref (base_symbol), NULL))), __temp231);
			ValaSymbol* __temp232 = NULL;
			(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), (__temp232 = vala_semantic_analyzer_symbol_lookup_inherited (base_symbol, vala_member_access_get_member_name (expr)))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
			(__temp232 == NULL ? NULL : (__temp232 = (g_object_unref (__temp232), NULL)));
		}
	}
	if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		char* __temp234 = NULL;
		char* __temp233 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp234 = g_strdup_printf ("The name `%s' does not exist in the context of `%s'", vala_member_access_get_member_name (expr), (__temp233 = vala_symbol_get_full_name (base_symbol)))));
		(__temp234 == NULL ? NULL : (__temp234 = (g_free (__temp234), NULL)));
		(__temp233 == NULL ? NULL : (__temp233 = (g_free (__temp233), NULL)));
		(base_symbol == NULL ? NULL : (base_symbol = (g_object_unref (base_symbol), NULL)));
		return;
	}
	ValaCodeNode* __temp235 = NULL;
	ValaCodeNode* member = (__temp235 = vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))), (__temp235 == NULL ? NULL : g_object_ref (__temp235)));
	ValaMemberAccessibility access = VALA_MEMBER_ACCESSIBILITY_PUBLIC;
	if (VALA_IS_FIELD (member)) {
		access = (VALA_FIELD (member))->access;
	} else {
		if (VALA_IS_METHOD (member)) {
			access = (VALA_METHOD (member))->access;
		}
	}
	if (access == VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
		ValaDataType* __temp236 = NULL;
		ValaDataType* target_type = (__temp236 = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (member)))), (__temp236 == NULL ? NULL : g_object_ref (__temp236)));
		ValaDataType* this_type = vala_semantic_analyzer_find_parent_type (self, self->priv->current_symbol);
		if (target_type != this_type) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			char* __temp238 = NULL;
			char* __temp237 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp238 = g_strdup_printf ("Access to private member `%s' denied", (__temp237 = vala_symbol_get_full_name (vala_code_node_get_symbol (member))))));
			(__temp238 == NULL ? NULL : (__temp238 = (g_free (__temp238), NULL)));
			(__temp237 == NULL ? NULL : (__temp237 = (g_free (__temp237), NULL)));
			(target_type == NULL ? NULL : (target_type = (g_object_unref (target_type), NULL)));
			(this_type == NULL ? NULL : (this_type = (g_object_unref (this_type), NULL)));
			(base_symbol == NULL ? NULL : (base_symbol = (g_object_unref (base_symbol), NULL)));
			(member == NULL ? NULL : (member = (g_object_unref (member), NULL)));
			return;
		}
		(target_type == NULL ? NULL : (target_type = (g_object_unref (target_type), NULL)));
		(this_type == NULL ? NULL : (this_type = (g_object_unref (this_type), NULL)));
	}
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	ValaTypeReference* __temp239 = NULL;
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp239 = vala_semantic_analyzer_get_static_type_for_node (self, vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)))))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	(__temp239 == NULL ? NULL : (__temp239 = (g_object_unref (__temp239), NULL)));
	(base_symbol == NULL ? NULL : (base_symbol = (g_object_unref (base_symbol), NULL)));
	(member == NULL ? NULL : (member = (g_object_unref (member), NULL)));
}


static gboolean vala_semantic_analyzer_is_type_compatible (ValaSemanticAnalyzer* self, ValaTypeReference* expression_type, ValaTypeReference* expected_type)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (expression_type), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (expected_type), FALSE);
	/* only null is compatible to null */
	if (vala_type_reference_get_data_type (expected_type) == NULL && vala_type_reference_get_type_parameter (expected_type) == NULL) {
		return (vala_type_reference_get_data_type (expression_type) == NULL && vala_type_reference_get_type_parameter (expected_type) == NULL);
	}
	if (vala_type_reference_get_data_type (expression_type) == NULL) {
		/* null can be cast to any reference or array type or pointer type */
		if (vala_type_reference_get_type_parameter (expected_type) != NULL || vala_data_type_is_reference_type (vala_type_reference_get_data_type (expected_type)) || vala_type_reference_get_reference_to_value_type (expected_type) || VALA_IS_POINTER (vala_type_reference_get_data_type (expected_type)) || VALA_IS_ARRAY (vala_type_reference_get_data_type (expected_type)) || VALA_IS_CALLBACK (vala_type_reference_get_data_type (expected_type)) || vala_type_reference_get_data_type (expected_type) == self->priv->pointer_type) {
			return TRUE;
		}
		return FALSE;
	}
	/* null is not compatible with any other type (i.e. value types) 
	 temporarily ignore type parameters */
	if (vala_type_reference_get_type_parameter (expected_type) != NULL) {
		return TRUE;
	}
	if (VALA_IS_ARRAY (vala_type_reference_get_data_type (expression_type)) != VALA_IS_ARRAY (vala_type_reference_get_data_type (expected_type))) {
		return FALSE;
	}
	if (VALA_IS_ENUM (vala_type_reference_get_data_type (expression_type)) && vala_type_reference_get_data_type (expected_type) == vala_type_reference_get_data_type (self->priv->int_type)) {
		return TRUE;
	}
	if (vala_type_reference_get_data_type (expression_type) == vala_type_reference_get_data_type (expected_type)) {
		return TRUE;
	}
	if (VALA_IS_STRUCT (vala_type_reference_get_data_type (expression_type)) && VALA_IS_STRUCT (vala_type_reference_get_data_type (expected_type))) {
		ValaStruct* __temp247 = NULL;
		ValaStruct* expr_struct = (__temp247 = VALA_STRUCT (vala_type_reference_get_data_type (expression_type)), (__temp247 == NULL ? NULL : g_object_ref (__temp247)));
		ValaStruct* __temp248 = NULL;
		ValaStruct* expect_struct = (__temp248 = VALA_STRUCT (vala_type_reference_get_data_type (expected_type)), (__temp248 == NULL ? NULL : g_object_ref (__temp248)));
		/* integer types may be implicitly cast to floating point types */
		if (vala_struct_is_integer_type (expr_struct) && vala_struct_is_floating_type (expect_struct)) {
			gboolean __temp249;
			return (__temp249 = TRUE, (expr_struct == NULL ? NULL : (expr_struct = (g_object_unref (expr_struct), NULL))), (expect_struct == NULL ? NULL : (expect_struct = (g_object_unref (expect_struct), NULL))), __temp249);
		}
		if ((vala_struct_is_integer_type (expr_struct) && vala_struct_is_integer_type (expect_struct)) || (vala_struct_is_floating_type (expr_struct) && vala_struct_is_floating_type (expect_struct))) {
			if (vala_struct_get_rank (expr_struct) <= vala_struct_get_rank (expect_struct)) {
				gboolean __temp250;
				return (__temp250 = TRUE, (expr_struct == NULL ? NULL : (expr_struct = (g_object_unref (expr_struct), NULL))), (expect_struct == NULL ? NULL : (expect_struct = (g_object_unref (expect_struct), NULL))), __temp250);
			}
		}
		(expr_struct == NULL ? NULL : (expr_struct = (g_object_unref (expr_struct), NULL)));
		(expect_struct == NULL ? NULL : (expect_struct = (g_object_unref (expect_struct), NULL)));
	}
	return vala_data_type_is_subtype_of (vala_type_reference_get_data_type (expression_type), vala_type_reference_get_data_type (expected_type));
}


static void vala_semantic_analyzer_real_visit_begin_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_invocation_expression_get_call (expr)))) {
		/* if method resolving didn't succeed, skip this check */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		return;
	}
	ValaSymbol* __temp252 = NULL;
	ValaSymbol* msym = (__temp252 = vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr)), (__temp252 == NULL ? NULL : g_object_ref (__temp252)));
	if (msym == NULL) {
		/* if no symbol found, skip this check */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
		return;
	}
	GList* params = NULL;
	if (VALA_IS_INVOKABLE (vala_symbol_get_node (msym))) {
		ValaInvokable* __temp253 = NULL;
		ValaInvokable* m = (__temp253 = VALA_INVOKABLE (vala_symbol_get_node (msym)), (__temp253 == NULL ? NULL : g_object_ref (__temp253)));
		if (vala_invokable_is_invokable (m)) {
			GList* __temp254 = NULL;
			params = (__temp254 = vala_invokable_get_parameters (m), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp254);
		} else {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "invocation not supported in this context");
			(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
			(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
			(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
			return;
		}
		(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
	} else {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "invocation not supported in this context");
		(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
		return;
	}
	GList* args = vala_invocation_expression_get_argument_list (expr);
	GList* arg_it = args;
	{
		GList* __temp256 = NULL;
		__temp256 = params;
		GList* param_it;
		for (param_it = __temp256; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				if (vala_formal_parameter_get_ellipsis (param)) {
					break;
				}
				if (arg_it != NULL) {
					ValaExpression* __temp255 = NULL;
					ValaExpression* arg = (__temp255 = VALA_EXPRESSION (arg_it->data), (__temp255 == NULL ? NULL : g_object_ref (__temp255)));
					/* store expected type for callback parameters */
					(vala_expression_set_expected_type (arg, vala_formal_parameter_get_type_reference (param)), vala_expression_get_expected_type (arg));
					arg_it = arg_it->next;
					(arg == NULL ? NULL : (arg = (g_object_unref (arg), NULL)));
				}
			}
		}
	}
	(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
}


static gboolean vala_semantic_analyzer_check_arguments (ValaSemanticAnalyzer* self, ValaExpression* expr, ValaSymbol* msym, GList* params, GList* args)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_EXPRESSION (expr), FALSE);
	g_return_val_if_fail (VALA_IS_SYMBOL (msym), FALSE);
	GList* prev_arg_it = NULL;
	GList* arg_it = args;
	ValaAttribute* __temp257 = NULL;
	gboolean __temp258;
	gboolean diag = (__temp258 = ((__temp257 = vala_code_node_get_attribute (vala_symbol_get_node (msym), "Diagnostics")) != NULL), (__temp257 == NULL ? NULL : (__temp257 = (g_object_unref (__temp257), NULL))), __temp258);
	gboolean ellipsis = FALSE;
	gint i = 0;
	{
		GList* __temp267 = NULL;
		__temp267 = params;
		GList* param_it;
		for (param_it = __temp267; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				if (vala_formal_parameter_get_ellipsis (param)) {
					ellipsis = TRUE;
					break;
				}
				/* header file necessary if we need to cast argument */
				if (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)) != NULL) {
					vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_formal_parameter_get_type_reference (param)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
				}
				if (arg_it == NULL) {
					if (vala_formal_parameter_get_default_expression (param) == NULL) {
						(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
						char* __temp260 = NULL;
						char* __temp259 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp260 = g_strdup_printf ("Too few arguments, method `%s' does not take %d arguments", (__temp259 = vala_symbol_get_full_name (msym)), g_list_length (args))));
						(__temp260 == NULL ? NULL : (__temp260 = (g_free (__temp260), NULL)));
						(__temp259 == NULL ? NULL : (__temp259 = (g_free (__temp259), NULL)));
						return FALSE;
					}
				} else {
					ValaExpression* __temp262 = NULL;
					ValaExpression* arg = (__temp262 = VALA_EXPRESSION (arg_it->data), (__temp262 == NULL ? NULL : g_object_ref (__temp262)));
					if (vala_expression_get_static_type (arg) != NULL && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (arg), vala_formal_parameter_get_type_reference (param))) {
						/* if there was an error in the argument,
						 * i.e. arg.static_type == null, skip type check */
						(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
						char* __temp265 = NULL;
						char* __temp264 = NULL;
						char* __temp263 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp265 = g_strdup_printf ("Argument %d: Cannot convert from `%s' to `%s'", i + 1, (__temp263 = vala_type_reference_to_string (vala_expression_get_static_type (arg))), (__temp264 = vala_type_reference_to_string (vala_formal_parameter_get_type_reference (param))))));
						(__temp265 == NULL ? NULL : (__temp265 = (g_free (__temp265), NULL)));
						(__temp264 == NULL ? NULL : (__temp264 = (g_free (__temp264), NULL)));
						(__temp263 == NULL ? NULL : (__temp263 = (g_free (__temp263), NULL)));
						gboolean __temp266;
						return (__temp266 = FALSE, (arg == NULL ? NULL : (arg = (g_object_unref (arg), NULL))), __temp266);
					}
					prev_arg_it = arg_it;
					arg_it = arg_it->next;
					i++;
					(arg == NULL ? NULL : (arg = (g_object_unref (arg), NULL)));
				}
			}
		}
	}
	if (!ellipsis && arg_it != NULL) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		char* __temp269 = NULL;
		char* __temp268 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp269 = g_strdup_printf ("Too many arguments, method `%s' does not take %d arguments", (__temp268 = vala_symbol_get_full_name (msym)), g_list_length (args))));
		(__temp269 == NULL ? NULL : (__temp269 = (g_free (__temp269), NULL)));
		(__temp268 == NULL ? NULL : (__temp268 = (g_free (__temp268), NULL)));
		return FALSE;
	}
	if (diag && prev_arg_it != NULL) {
		ValaExpression* __temp271 = NULL;
		ValaExpression* format_arg = (__temp271 = VALA_EXPRESSION (prev_arg_it->data), (__temp271 == NULL ? NULL : g_object_ref (__temp271)));
		if (VALA_IS_LITERAL_EXPRESSION (format_arg)) {
			ValaStringLiteral* __temp272 = NULL;
			ValaStringLiteral* format_lit = (__temp272 = VALA_STRING_LITERAL (vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (format_arg)))), (__temp272 == NULL ? NULL : g_object_ref (__temp272)));
			char* __temp273 = NULL;
			(vala_string_literal_set_value (format_lit, (__temp273 = g_strdup_printf ("\"%s:%d: %s", vala_source_file_get_filename (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)))), vala_source_reference_get_first_line (vala_code_node_get_source_reference (VALA_CODE_NODE (expr))), g_utf8_offset_to_pointer (vala_string_literal_get_value (format_lit), 1)))), vala_string_literal_get_value (format_lit));
			(__temp273 == NULL ? NULL : (__temp273 = (g_free (__temp273), NULL)));
			(format_lit == NULL ? NULL : (format_lit = (g_object_unref (format_lit), NULL)));
		}
		(format_arg == NULL ? NULL : (format_arg = (g_object_unref (format_arg), NULL)));
	}
	return TRUE;
}


static void vala_semantic_analyzer_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (expr))) {
		return;
	}
	ValaSymbol* __temp275 = NULL;
	ValaSymbol* msym = (__temp275 = vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr)), (__temp275 == NULL ? NULL : g_object_ref (__temp275)));
	ValaTypeReference* ret_type = NULL;
	GList* params = NULL;
	if (VALA_IS_INVOKABLE (vala_symbol_get_node (msym))) {
		ValaInvokable* __temp276 = NULL;
		ValaInvokable* m = (__temp276 = VALA_INVOKABLE (vala_symbol_get_node (msym)), (__temp276 == NULL ? NULL : g_object_ref (__temp276)));
		ValaTypeReference* __temp277 = NULL;
		ret_type = (__temp277 = vala_invokable_get_return_type (m), (ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL))), __temp277);
		GList* __temp278 = NULL;
		params = (__temp278 = vala_invokable_get_parameters (m), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp278);
		/* resolve generic return values*/
		if (vala_type_reference_get_type_parameter (ret_type) != NULL) {
			if (!(VALA_IS_MEMBER_ACCESS (vala_invocation_expression_get_call (expr)))) {
				vala_report_error (vala_code_node_get_source_reference ((VALA_CODE_NODE (m))), "internal error: unsupported generic return value");
				(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
				(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
				(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
				(ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL)));
				(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
				return;
			}
			ValaMemberAccess* __temp279 = NULL;
			ValaMemberAccess* ma = (__temp279 = VALA_MEMBER_ACCESS (vala_invocation_expression_get_call (expr)), (__temp279 == NULL ? NULL : g_object_ref (__temp279)));
			if (vala_member_access_get_inner (ma) == NULL) {
				/* TODO resolve generic return values within the type hierarchy if possible*/
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "internal error: resolving generic return values within type hierarchy not supported yet");
				(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
				(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
				(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
				(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
				(ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL)));
				(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
				return;
			} else {
				ValaTypeReference* __temp280 = NULL;
				ValaTypeReference* instance_type = (__temp280 = vala_expression_get_static_type (vala_member_access_get_inner (ma)), (__temp280 == NULL ? NULL : g_object_ref (__temp280)));
				while (VALA_CODE_NODE (vala_type_reference_get_data_type (instance_type)) != vala_symbol_get_node (vala_symbol_get_parent_symbol (msym))) {
					/* trace type arguments back to the datatype where the method has been declared*/
					GList* base_types = NULL;
					if (VALA_IS_CLASS (vala_type_reference_get_data_type (instance_type))) {
						ValaClass* __temp281 = NULL;
						ValaClass* cl = (__temp281 = VALA_CLASS (vala_type_reference_get_data_type (instance_type)), (__temp281 == NULL ? NULL : g_object_ref (__temp281)));
						GList* __temp282 = NULL;
						base_types = (__temp282 = vala_class_get_base_types (cl), (base_types == NULL ? NULL : (base_types = (g_list_free (base_types), NULL))), __temp282);
						(cl == NULL ? NULL : (cl = (g_object_unref (cl), NULL)));
					} else {
						if (VALA_IS_INTERFACE (vala_type_reference_get_data_type (instance_type))) {
							ValaInterface* __temp283 = NULL;
							ValaInterface* iface = (__temp283 = VALA_INTERFACE (vala_type_reference_get_data_type (instance_type)), (__temp283 == NULL ? NULL : g_object_ref (__temp283)));
							GList* __temp284 = NULL;
							base_types = (__temp284 = vala_interface_get_prerequisites (iface), (base_types == NULL ? NULL : (base_types = (g_list_free (base_types), NULL))), __temp284);
							(iface == NULL ? NULL : (iface = (g_object_unref (iface), NULL)));
						} else {
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "internal error: unsupported generic type");
							(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
							(base_types == NULL ? NULL : (base_types = (g_list_free (base_types), NULL)));
							(instance_type == NULL ? NULL : (instance_type = (g_object_unref (instance_type), NULL)));
							(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
							(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
							(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
							(ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL)));
							(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
							return;
						}
					}
					{
						GList* __temp292 = NULL;
						__temp292 = base_types;
						GList* base_type_it;
						for (base_type_it = __temp292; base_type_it != NULL; base_type_it = base_type_it->next) {
							ValaTypeReference* base_type = base_type_it->data;
							{
								ValaSymbol* __temp285 = NULL;
								gboolean __temp286;
								if ((__temp286 = (__temp285 = vala_semantic_analyzer_symbol_lookup_inherited (vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (base_type))), vala_symbol_get_name (msym))) != NULL, (__temp285 == NULL ? NULL : (__temp285 = (g_object_unref (__temp285), NULL))), __temp286)) {
									/* construct a new type reference for the base type with correctly linked type arguments*/
									ValaTypeReference* instance_base_type = vala_type_reference_new ();
									(vala_type_reference_set_data_type (instance_base_type, vala_type_reference_get_data_type (base_type)), vala_type_reference_get_data_type (instance_base_type));
									{
										GList* __temp289 = NULL;
										__temp289 = vala_type_reference_get_type_arguments (base_type);
										GList* type_arg_it;
										for (type_arg_it = __temp289; type_arg_it != NULL; type_arg_it = type_arg_it->next) {
											ValaTypeReference* type_arg = type_arg_it->data;
											{
												if (vala_type_reference_get_type_parameter (type_arg) != NULL) {
													/* link to type argument of derived type*/
													gint param_index = vala_data_type_get_type_parameter_index (vala_type_reference_get_data_type (instance_type), vala_type_parameter_get_name (vala_type_reference_get_type_parameter (type_arg)));
													if (param_index == -1) {
														char* __temp287 = NULL;
														vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp287 = g_strdup_printf ("internal error: unknown type parameter %s", vala_type_parameter_get_name (vala_type_reference_get_type_parameter (type_arg)))));
														(__temp287 == NULL ? NULL : (__temp287 = (g_free (__temp287), NULL)));
														(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
														(instance_base_type == NULL ? NULL : (instance_base_type = (g_object_unref (instance_base_type), NULL)));
														(base_types == NULL ? NULL : (base_types = (g_list_free (base_types), NULL)));
														(instance_type == NULL ? NULL : (instance_type = (g_object_unref (instance_type), NULL)));
														(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
														(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
														(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
														(ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL)));
														(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
														return;
													}
													GList* __temp288 = NULL;
													type_arg = g_list_nth_data ((__temp288 = vala_type_reference_get_type_arguments (instance_type)), param_index);
													(__temp288 == NULL ? NULL : (__temp288 = (g_list_free (__temp288), NULL)));
												}
												vala_type_reference_add_type_argument (instance_base_type, type_arg);
											}
										}
										(__temp289 == NULL ? NULL : (__temp289 = (g_list_free (__temp289), NULL)));
									}
									ValaTypeReference* __temp291 = NULL;
									ValaTypeReference* __temp290 = NULL;
									instance_type = (__temp291 = (__temp290 = instance_base_type, (__temp290 == NULL ? NULL : g_object_ref (__temp290))), (instance_type == NULL ? NULL : (instance_type = (g_object_unref (instance_type), NULL))), __temp291);
									(instance_base_type == NULL ? NULL : (instance_base_type = (g_object_unref (instance_base_type), NULL)));
								}
							}
						}
					}
					(base_types == NULL ? NULL : (base_types = (g_list_free (base_types), NULL)));
				}
				if (VALA_CODE_NODE (vala_type_reference_get_data_type (instance_type)) != vala_symbol_get_node (vala_symbol_get_parent_symbol (msym))) {
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "internal error: generic type parameter tracing not supported yet");
					(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
					(instance_type == NULL ? NULL : (instance_type = (g_object_unref (instance_type), NULL)));
					(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
					(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
					(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
					(ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL)));
					(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
					return;
				}
				gint param_index = vala_data_type_get_type_parameter_index (vala_type_reference_get_data_type (instance_type), vala_type_parameter_get_name (vala_type_reference_get_type_parameter (ret_type)));
				if (param_index == -1) {
					char* __temp293 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp293 = g_strdup_printf ("internal error: unknown type parameter %s", vala_type_parameter_get_name (vala_type_reference_get_type_parameter (ret_type)))));
					(__temp293 == NULL ? NULL : (__temp293 = (g_free (__temp293), NULL)));
					(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
					(instance_type == NULL ? NULL : (instance_type = (g_object_unref (instance_type), NULL)));
					(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
					(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
					(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
					(ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL)));
					(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
					return;
				}
				ValaTypeReference* __temp296 = NULL;
				ValaTypeReference* __temp295 = NULL;
				GList* __temp294 = NULL;
				ret_type = (__temp296 = (__temp295 = VALA_TYPE_REFERENCE (g_list_nth_data ((__temp294 = vala_type_reference_get_type_arguments (instance_type)), param_index)), (__temp295 == NULL ? NULL : g_object_ref (__temp295))), (ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL))), __temp296);
				(__temp294 == NULL ? NULL : (__temp294 = (g_list_free (__temp294), NULL)));
				if (ret_type == NULL) {
					char* __temp297 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp297 = g_strdup_printf ("internal error: no actual argument found for type parameter %s", vala_type_parameter_get_name (vala_type_reference_get_type_parameter (ret_type)))));
					(__temp297 == NULL ? NULL : (__temp297 = (g_free (__temp297), NULL)));
					(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
					(instance_type == NULL ? NULL : (instance_type = (g_object_unref (instance_type), NULL)));
					(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
					(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
					(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
					(ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL)));
					(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
					return;
				}
				(instance_type == NULL ? NULL : (instance_type = (g_object_unref (instance_type), NULL)));
			}
			(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
		}
		(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
	}
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), ret_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	GList* __temp298 = NULL;
	vala_semantic_analyzer_check_arguments (self, VALA_EXPRESSION (expr), msym, params, (__temp298 = vala_invocation_expression_get_argument_list (expr)));
	(__temp298 == NULL ? NULL : (__temp298 = (g_list_free (__temp298), NULL)));
	(msym == NULL ? NULL : (msym = (g_object_unref (msym), NULL)));
	(ret_type == NULL ? NULL : (ret_type = (g_object_unref (ret_type), NULL)));
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
}


static void vala_semantic_analyzer_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (expr));
	if (vala_expression_get_static_type (vala_element_access_get_container (expr)) == NULL) {
		/* don't proceed if a child expression failed */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		return;
	}
	/* assign a static_type when possible */
	if (VALA_IS_ARRAY (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_element_access_get_container (expr))))) {
		GList* args = vala_type_reference_get_type_arguments (vala_expression_get_static_type (vala_element_access_get_container (expr)));
		if (g_list_length (args) != 1) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "internal error: array reference without type arguments");
			(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
			return;
		}
		(vala_expression_set_static_type (VALA_EXPRESSION (expr), VALA_TYPE_REFERENCE (args->data)), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
		(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
	} else {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		char* __temp300 = NULL;
		char* __temp299 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp300 = g_strdup_printf ("The expression `%s' does not denote an Array", (__temp299 = vala_type_reference_to_string (vala_expression_get_static_type (vala_element_access_get_container (expr)))))));
		(__temp300 == NULL ? NULL : (__temp300 = (g_free (__temp300), NULL)));
		(__temp299 == NULL ? NULL : (__temp299 = (g_free (__temp299), NULL)));
	}
	{
		GList* __temp301 = NULL;
		__temp301 = vala_element_access_get_indices (expr);
		GList* e_it;
		for (e_it = __temp301; e_it != NULL; e_it = e_it->next) {
			ValaExpression* e = e_it->data;
			{
				/* check if the index is of type integer 
				 don't proceed if a child expression failed */
				if (vala_expression_get_static_type (e) == NULL) {
					return;
				}
				/* check if the index is of type integer */
				if (vala_type_reference_get_data_type (vala_expression_get_static_type (e)) != vala_type_reference_get_data_type (self->priv->int_type)) {
					(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (e)), "Expression of type `int' expected");
				}
			}
		}
		(__temp301 == NULL ? NULL : (__temp301 = (g_list_free (__temp301), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_base_access (ValaCodeVisitor* base, ValaBaseAccess* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BASE_ACCESS (expr));
	if (self->priv->current_class == NULL) {
		if (self->priv->current_struct == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Base access invalid outside of class and struct");
			return;
		} else {
			GList* __temp302 = NULL;
			gboolean __temp303;
			if ((__temp303 = g_list_length ((__temp302 = vala_struct_get_base_types (self->priv->current_struct))) != 1, (__temp302 == NULL ? NULL : (__temp302 = (g_list_free (__temp302), NULL))), __temp303)) {
				(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
				char* __temp305 = NULL;
				GList* __temp304 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp305 = g_strdup_printf ("Base access invalid without base type %d", g_list_length ((__temp304 = vala_struct_get_base_types (self->priv->current_struct))))));
				(__temp305 == NULL ? NULL : (__temp305 = (g_free (__temp305), NULL)));
				(__temp304 == NULL ? NULL : (__temp304 = (g_list_free (__temp304), NULL)));
				return;
			}
		}
		GList* __temp306 = NULL;
		(vala_expression_set_static_type (VALA_EXPRESSION (expr), g_list_first ((__temp306 = vala_struct_get_base_types (self->priv->current_struct)))->data), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
		(__temp306 == NULL ? NULL : (__temp306 = (g_list_free (__temp306), NULL)));
	} else {
		ValaTypeReference* __temp307 = NULL;
		(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp307 = vala_type_reference_new ())), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
		(__temp307 == NULL ? NULL : (__temp307 = (g_object_unref (__temp307), NULL)));
		(vala_type_reference_set_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)), VALA_DATA_TYPE (vala_class_get_base_class (self->priv->current_class))), vala_type_reference_get_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	}
	(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)))))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
}


static void vala_semantic_analyzer_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_POSTFIX_EXPRESSION (expr));
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_postfix_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static void vala_semantic_analyzer_real_visit_end_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_OBJECT_CREATION_EXPRESSION (expr));
	ValaDataType* type = NULL;
	if (vala_object_creation_expression_get_type_reference (expr) == NULL) {
		if (vala_object_creation_expression_get_member_name (expr) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Incomplete object creation expression");
			(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
			return;
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (vala_object_creation_expression_get_member_name (expr))) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
			return;
		}
		ValaCodeNode* __temp308 = NULL;
		ValaCodeNode* constructor_node = (__temp308 = vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (vala_object_creation_expression_get_member_name (expr)))), (__temp308 == NULL ? NULL : g_object_ref (__temp308)));
		ValaCodeNode* __temp309 = NULL;
		ValaCodeNode* type_node = (__temp309 = vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (vala_object_creation_expression_get_member_name (expr)))), (__temp309 == NULL ? NULL : g_object_ref (__temp309)));
		GList* type_args = vala_member_access_get_type_arguments (vala_object_creation_expression_get_member_name (expr));
		if (VALA_IS_METHOD (constructor_node)) {
			ValaCodeNode* __temp311 = NULL;
			ValaCodeNode* __temp310 = NULL;
			type_node = (__temp311 = (__temp310 = vala_symbol_get_node (vala_symbol_get_parent_symbol (vala_code_node_get_symbol (constructor_node))), (__temp310 == NULL ? NULL : g_object_ref (__temp310))), (type_node == NULL ? NULL : (type_node = (g_object_unref (type_node), NULL))), __temp311);
			ValaMethod* __temp312 = NULL;
			ValaMethod* constructor = (__temp312 = VALA_METHOD (constructor_node), (__temp312 == NULL ? NULL : g_object_ref (__temp312)));
			if (!(VALA_IS_CREATION_METHOD (constructor_node))) {
				(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
				char* __temp314 = NULL;
				char* __temp313 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp314 = g_strdup_printf ("`%s' is not a creation method", (__temp313 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (constructor)))))));
				(__temp314 == NULL ? NULL : (__temp314 = (g_free (__temp314), NULL)));
				(__temp313 == NULL ? NULL : (__temp313 = (g_free (__temp313), NULL)));
				(constructor == NULL ? NULL : (constructor = (g_object_unref (constructor), NULL)));
				(constructor_node == NULL ? NULL : (constructor_node = (g_object_unref (constructor_node), NULL)));
				(type_node == NULL ? NULL : (type_node = (g_object_unref (type_node), NULL)));
				(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
				(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
				return;
			}
			(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (constructor))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
			GList* __temp315 = NULL;
			type_args = (__temp315 = vala_member_access_get_type_arguments ((VALA_MEMBER_ACCESS (vala_member_access_get_inner (vala_object_creation_expression_get_member_name (expr))))), (type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL))), __temp315);
			(constructor == NULL ? NULL : (constructor = (g_object_unref (constructor), NULL)));
		}
		if (VALA_IS_CLASS (type_node) || VALA_IS_STRUCT (type_node)) {
			ValaDataType* __temp317 = NULL;
			ValaDataType* __temp316 = NULL;
			type = (__temp317 = (__temp316 = VALA_DATA_TYPE (type_node), (__temp316 == NULL ? NULL : g_object_ref (__temp316))), (type == NULL ? NULL : (type = (g_object_unref (type), NULL))), __temp317);
		} else {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			char* __temp319 = NULL;
			char* __temp318 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp319 = g_strdup_printf ("`%s' is not a class or struct", (__temp318 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (type)))))));
			(__temp319 == NULL ? NULL : (__temp319 = (g_free (__temp319), NULL)));
			(__temp318 == NULL ? NULL : (__temp318 = (g_free (__temp318), NULL)));
			(constructor_node == NULL ? NULL : (constructor_node = (g_object_unref (constructor_node), NULL)));
			(type_node == NULL ? NULL : (type_node = (g_object_unref (type_node), NULL)));
			(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
			(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
			return;
		}
		ValaTypeReference* __temp320 = NULL;
		(vala_object_creation_expression_set_type_reference (expr, (__temp320 = vala_type_reference_new ())), vala_object_creation_expression_get_type_reference (expr));
		(__temp320 == NULL ? NULL : (__temp320 = (g_object_unref (__temp320), NULL)));
		(vala_type_reference_set_data_type (vala_object_creation_expression_get_type_reference (expr), type), vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr)));
		{
			GList* __temp321 = NULL;
			__temp321 = type_args;
			GList* type_arg_it;
			for (type_arg_it = __temp321; type_arg_it != NULL; type_arg_it = type_arg_it->next) {
				ValaTypeReference* type_arg = type_arg_it->data;
				{
					vala_type_reference_add_type_argument (vala_object_creation_expression_get_type_reference (expr), type_arg);
				}
			}
		}
		(constructor_node == NULL ? NULL : (constructor_node = (g_object_unref (constructor_node), NULL)));
		(type_node == NULL ? NULL : (type_node = (g_object_unref (type_node), NULL)));
		(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
	} else {
		ValaDataType* __temp323 = NULL;
		ValaDataType* __temp322 = NULL;
		type = (__temp323 = (__temp322 = vala_type_reference_get_data_type (vala_object_creation_expression_get_type_reference (expr)), (__temp322 == NULL ? NULL : g_object_ref (__temp322))), (type == NULL ? NULL : (type = (g_object_unref (type), NULL))), __temp323);
	}
	if (!vala_data_type_is_reference_type (type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		char* __temp325 = NULL;
		char* __temp324 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp325 = g_strdup_printf ("Can't create instance of value type `%s'", (__temp324 = vala_type_reference_to_string (vala_object_creation_expression_get_type_reference (expr))))));
		(__temp325 == NULL ? NULL : (__temp325 = (g_free (__temp325), NULL)));
		(__temp324 == NULL ? NULL : (__temp324 = (g_free (__temp324), NULL)));
		(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
		return;
	}
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	ValaTypeReference* __temp326 = NULL;
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp326 = vala_type_reference_copy (vala_object_creation_expression_get_type_reference (expr)))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	(__temp326 == NULL ? NULL : (__temp326 = (g_object_unref (__temp326), NULL)));
	(vala_type_reference_set_transfers_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE), vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	if (VALA_IS_CLASS (type)) {
		ValaClass* __temp327 = NULL;
		ValaClass* cl = (__temp327 = VALA_CLASS (type), (__temp327 == NULL ? NULL : g_object_ref (__temp327)));
		if (vala_class_get_is_abstract (cl)) {
			(vala_expression_set_static_type (VALA_EXPRESSION (expr), NULL), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			char* __temp329 = NULL;
			char* __temp328 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp329 = g_strdup_printf ("Can't create instance of abstract class `%s'", (__temp328 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cl)))))));
			(__temp329 == NULL ? NULL : (__temp329 = (g_free (__temp329), NULL)));
			(__temp328 == NULL ? NULL : (__temp328 = (g_free (__temp328), NULL)));
			(cl == NULL ? NULL : (cl = (g_object_unref (cl), NULL)));
			(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
			return;
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && vala_class_get_default_construction_method (cl) != NULL) {
			(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (vala_class_get_default_construction_method (cl)))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
		}
		while (cl != NULL) {
			if (VALA_DATA_TYPE (cl) == self->priv->initially_unowned_type) {
				(vala_type_reference_set_floating_reference (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE), vala_type_reference_get_floating_reference (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
				break;
			}
			ValaClass* __temp331 = NULL;
			ValaClass* __temp330 = NULL;
			cl = (__temp331 = (__temp330 = vala_class_get_base_class (cl), (__temp330 == NULL ? NULL : g_object_ref (__temp330))), (cl == NULL ? NULL : (cl = (g_object_unref (cl), NULL))), __temp331);
		}
		(cl == NULL ? NULL : (cl = (g_object_unref (cl), NULL)));
	} else {
		if (VALA_IS_STRUCT (type)) {
			ValaStruct* __temp332 = NULL;
			ValaStruct* st = (__temp332 = VALA_STRUCT (type), (__temp332 == NULL ? NULL : g_object_ref (__temp332)));
			if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && vala_struct_get_default_construction_method (st) != NULL) {
				(vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (vala_struct_get_default_construction_method (st)))), vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
			}
			(st == NULL ? NULL : (st = (g_object_unref (st), NULL)));
		}
	}
	GList* __temp333 = NULL;
	gboolean __temp334;
	if ((__temp334 = vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && g_list_length ((__temp333 = vala_object_creation_expression_get_argument_list (expr))) != 0, (__temp333 == NULL ? NULL : (__temp333 = (g_list_free (__temp333), NULL))), __temp334)) {
		(vala_expression_set_static_type (VALA_EXPRESSION (expr), NULL), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		char* __temp336 = NULL;
		char* __temp335 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp336 = g_strdup_printf ("No arguments allowed when constructing type `%s'", (__temp335 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (type)))))));
		(__temp336 == NULL ? NULL : (__temp336 = (g_free (__temp336), NULL)));
		(__temp335 == NULL ? NULL : (__temp335 = (g_free (__temp335), NULL)));
		(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
		return;
	}
	if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) != NULL) {
		ValaMethod* __temp337 = NULL;
		ValaMethod* m = (__temp337 = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)))), (__temp337 == NULL ? NULL : g_object_ref (__temp337)));
		GList* __temp339 = NULL;
		GList* __temp338 = NULL;
		vala_semantic_analyzer_check_arguments (self, VALA_EXPRESSION (expr), vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp338 = vala_invokable_get_parameters (VALA_INVOKABLE (m))), (__temp339 = vala_object_creation_expression_get_argument_list (expr)));
		(__temp339 == NULL ? NULL : (__temp339 = (g_list_free (__temp339), NULL)));
		(__temp338 == NULL ? NULL : (__temp338 = (g_list_free (__temp338), NULL)));
		(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
	}
	(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
}


static void vala_semantic_analyzer_real_visit_typeof_expression (ValaCodeVisitor* base, ValaTypeofExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_TYPEOF_EXPRESSION (expr));
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->type_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static gboolean vala_semantic_analyzer_is_numeric_type (ValaSemanticAnalyzer* self, ValaTypeReference* type)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), FALSE);
	if (!(VALA_IS_STRUCT (vala_type_reference_get_data_type (type)))) {
		return FALSE;
	}
	ValaStruct* __temp341 = NULL;
	ValaStruct* st = (__temp341 = VALA_STRUCT (vala_type_reference_get_data_type (type)), (__temp341 == NULL ? NULL : g_object_ref (__temp341)));
	gboolean __temp342;
	return (__temp342 = vala_struct_is_integer_type (st) || vala_struct_is_floating_type (st), (st == NULL ? NULL : (st = (g_object_unref (st), NULL))), __temp342);
	(st == NULL ? NULL : (st = (g_object_unref (st), NULL)));
}


static gboolean vala_semantic_analyzer_is_integer_type (ValaSemanticAnalyzer* self, ValaTypeReference* type)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), FALSE);
	if (!(VALA_IS_STRUCT (vala_type_reference_get_data_type (type)))) {
		return FALSE;
	}
	ValaStruct* __temp344 = NULL;
	ValaStruct* st = (__temp344 = VALA_STRUCT (vala_type_reference_get_data_type (type)), (__temp344 == NULL ? NULL : g_object_ref (__temp344)));
	gboolean __temp345;
	return (__temp345 = vala_struct_is_integer_type (st), (st == NULL ? NULL : (st = (g_object_unref (st), NULL))), __temp345);
	(st == NULL ? NULL : (st = (g_object_unref (st), NULL)));
}


static void vala_semantic_analyzer_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_UNARY_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_unary_expression_get_inner (expr)))) {
		/* if there was an error in the inner expression, skip type check */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		return;
	}
	if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_PLUS || vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_MINUS) {
		/* integer or floating point type*/
		if (!vala_semantic_analyzer_is_numeric_type (self, vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			char* __temp347 = NULL;
			char* __temp346 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp347 = g_strdup_printf ("Operator not supported for `%s'", (__temp346 = vala_type_reference_to_string (vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))))));
			(__temp347 == NULL ? NULL : (__temp347 = (g_free (__temp347), NULL)));
			(__temp346 == NULL ? NULL : (__temp346 = (g_free (__temp346), NULL)));
			return;
		}
		(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	} else {
		if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_LOGICAL_NEGATION) {
			/* boolean type*/
			if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_unary_expression_get_inner (expr))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
				(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
				char* __temp349 = NULL;
				char* __temp348 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp349 = g_strdup_printf ("Operator not supported for `%s'", (__temp348 = vala_type_reference_to_string (vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))))));
				(__temp349 == NULL ? NULL : (__temp349 = (g_free (__temp349), NULL)));
				(__temp348 == NULL ? NULL : (__temp348 = (g_free (__temp348), NULL)));
				return;
			}
			(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
		} else {
			if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_BITWISE_COMPLEMENT) {
				/* integer type*/
				if (!vala_semantic_analyzer_is_integer_type (self, vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))) {
					(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
					char* __temp351 = NULL;
					char* __temp350 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp351 = g_strdup_printf ("Operator not supported for `%s'", (__temp350 = vala_type_reference_to_string (vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))))));
					(__temp351 == NULL ? NULL : (__temp351 = (g_free (__temp351), NULL)));
					(__temp350 == NULL ? NULL : (__temp350 = (g_free (__temp350), NULL)));
					return;
				}
				(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
			} else {
				if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_INCREMENT || vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_DECREMENT) {
					/* integer type*/
					if (!vala_semantic_analyzer_is_integer_type (self, vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))) {
						(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
						char* __temp353 = NULL;
						char* __temp352 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp353 = g_strdup_printf ("Operator not supported for `%s'", (__temp352 = vala_type_reference_to_string (vala_expression_get_static_type (vala_unary_expression_get_inner (expr)))))));
						(__temp353 == NULL ? NULL : (__temp353 = (g_free (__temp353), NULL)));
						(__temp352 == NULL ? NULL : (__temp352 = (g_free (__temp352), NULL)));
						return;
					}
					ValaMemberAccess* ma = vala_semantic_analyzer_find_member_access (self, vala_unary_expression_get_inner (expr));
					if (ma == NULL) {
						(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Prefix operators not supported for this expression");
						(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
						return;
					}
					ValaMemberAccess* old_value = vala_member_access_new (vala_member_access_get_inner (ma), vala_member_access_get_member_name (ma), NULL);
					ValaBinaryExpression* bin = vala_binary_expression_new ((vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_INCREMENT ? VALA_BINARY_OPERATOR_PLUS : VALA_BINARY_OPERATOR_MINUS), VALA_EXPRESSION (old_value), VALA_EXPRESSION (vala_literal_expression_new (VALA_LITERAL (vala_integer_literal_new ("1", NULL)), NULL)), NULL);
					ValaAssignment* assignment = vala_assignment_new (VALA_EXPRESSION (ma), VALA_EXPRESSION (bin), VALA_ASSIGNMENT_OPERATOR_SIMPLE, NULL);
					vala_code_node_replace (vala_code_node_get_parent_node (VALA_CODE_NODE (expr)), VALA_CODE_NODE (expr), VALA_CODE_NODE (assignment));
					vala_code_node_accept (VALA_CODE_NODE (assignment), VALA_CODE_VISITOR (self));
					(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
					(old_value == NULL ? NULL : (old_value = (g_object_unref (old_value), NULL)));
					(bin == NULL ? NULL : (bin = (g_object_unref (bin), NULL)));
					(assignment == NULL ? NULL : (assignment = (g_object_unref (assignment), NULL)));
					return;
					(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
					(old_value == NULL ? NULL : (old_value = (g_object_unref (old_value), NULL)));
					(bin == NULL ? NULL : (bin = (g_object_unref (bin), NULL)));
					(assignment == NULL ? NULL : (assignment = (g_object_unref (assignment), NULL)));
				} else {
					if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_REF) {
						/* value type*/
						(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
					} else {
						if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_OUT) {
							/* reference type*/
							(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
						} else {
							(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "internal error: unsupported unary operator");
							return;
						}
					}
				}
			}
		}
	}
}


static ValaMemberAccess* vala_semantic_analyzer_find_member_access (ValaSemanticAnalyzer* self, ValaExpression* expr)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_EXPRESSION (expr), NULL);
	if (VALA_IS_PARENTHESIZED_EXPRESSION (expr)) {
		ValaParenthesizedExpression* __temp354 = NULL;
		ValaParenthesizedExpression* pe = (__temp354 = VALA_PARENTHESIZED_EXPRESSION (expr), (__temp354 == NULL ? NULL : g_object_ref (__temp354)));
		ValaMemberAccess* __temp355 = NULL;
		return (__temp355 = vala_semantic_analyzer_find_member_access (self, vala_parenthesized_expression_get_inner (pe)), (pe == NULL ? NULL : (pe = (g_object_unref (pe), NULL))), __temp355);
		(pe == NULL ? NULL : (pe = (g_object_unref (pe), NULL)));
	}
	if (VALA_IS_MEMBER_ACCESS (expr)) {
		ValaMemberAccess* __temp356 = NULL;
		return (__temp356 = VALA_MEMBER_ACCESS (expr), (__temp356 == NULL ? NULL : g_object_ref (__temp356)));
	}
	return VALA_MEMBER_ACCESS (NULL);
}


static void vala_semantic_analyzer_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CAST_EXPRESSION (expr));
	if (vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr)) == NULL) {
		return;
	}
	/* if type resolving didn't succeed, skip this check 
	 FIXME: check whether cast is allowed*/
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_cast_expression_get_type_reference (expr)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_cast_expression_get_type_reference (expr)), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static void vala_semantic_analyzer_real_visit_pointer_indirection (ValaCodeVisitor* base, ValaPointerIndirection* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_POINTER_INDIRECTION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_pointer_indirection_get_inner (expr)))) {
		return;
	}
	if (vala_expression_get_static_type (vala_pointer_indirection_get_inner (expr)) == NULL) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "internal error: unknown type of inner expression");
		return;
	}
	if (!(VALA_IS_POINTER (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_pointer_indirection_get_inner (expr)))))) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Pointer indirection not supported for this expression");
		return;
	}
	ValaPointer* __temp359 = NULL;
	ValaPointer* pointer = (__temp359 = VALA_POINTER (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_pointer_indirection_get_inner (expr)))), (__temp359 == NULL ? NULL : g_object_ref (__temp359)));
	ValaTypeReference* __temp360 = NULL;
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp360 = vala_type_reference_new ())), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	(__temp360 == NULL ? NULL : (__temp360 = (g_object_unref (__temp360), NULL)));
	(vala_type_reference_set_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)), vala_pointer_get_referent_type (pointer)), vala_type_reference_get_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	(vala_type_reference_set_takes_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr)), vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_pointer_indirection_get_inner (expr)))), vala_type_reference_get_takes_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	(pointer == NULL ? NULL : (pointer = (g_object_unref (pointer), NULL)));
}


static void vala_semantic_analyzer_real_visit_addressof_expression (ValaCodeVisitor* base, ValaAddressofExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ADDRESSOF_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_addressof_expression_get_inner (expr)))) {
		return;
	}
	if (vala_expression_get_static_type (vala_addressof_expression_get_inner (expr)) == NULL) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "internal error: unknown type of inner expression");
		return;
	}
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_addressof_expression_get_inner (expr))) == NULL) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Address-of operator not supported for this expression");
		return;
	}
	ValaTypeReference* __temp361 = NULL;
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp361 = vala_type_reference_new ())), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
	(__temp361 == NULL ? NULL : (__temp361 = (g_object_unref (__temp361), NULL)));
	ValaPointer* __temp362 = NULL;
	(vala_type_reference_set_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr)), VALA_DATA_TYPE ((__temp362 = vala_data_type_get_pointer (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_addressof_expression_get_inner (expr))))))), vala_type_reference_get_data_type (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
	(__temp362 == NULL ? NULL : (__temp362 = (g_object_unref (__temp362), NULL)));
	(vala_type_reference_set_takes_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr)), vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_addressof_expression_get_inner (expr)))), vala_type_reference_get_takes_ownership (vala_expression_get_static_type (VALA_EXPRESSION (expr))));
}


static gboolean vala_semantic_analyzer_check_binary_type (ValaSemanticAnalyzer* self, ValaBinaryExpression* expr, char* operation)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_BINARY_EXPRESSION (expr), FALSE);
	g_return_val_if_fail (operation != NULL, FALSE);
	if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_right (expr)), vala_expression_get_static_type (vala_binary_expression_get_left (expr)))) {
		char* __temp365 = NULL;
		char* __temp364 = NULL;
		char* __temp363 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp365 = g_strdup_printf ("%s: Cannot convert from `%s' to `%s'", operation, (__temp363 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))), (__temp364 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))))));
		(__temp365 == NULL ? NULL : (__temp365 = (g_free (__temp365), NULL)));
		(__temp364 == NULL ? NULL : (__temp364 = (g_free (__temp364), NULL)));
		(__temp363 == NULL ? NULL : (__temp363 = (g_free (__temp363), NULL)));
		return FALSE;
	}
	return TRUE;
}


static ValaTypeReference* vala_semantic_analyzer_get_arithmetic_result_type (ValaSemanticAnalyzer* self, ValaTypeReference* left_type, ValaTypeReference* right_type)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (left_type), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (right_type), NULL);
	if (!(VALA_IS_STRUCT (vala_type_reference_get_data_type (left_type))) || !(VALA_IS_STRUCT (vala_type_reference_get_data_type (right_type)))) {
		return VALA_TYPE_REFERENCE (NULL);
	}
	/* at least one operand not struct*/
	ValaStruct* __temp369 = NULL;
	ValaStruct* left = (__temp369 = VALA_STRUCT (vala_type_reference_get_data_type (left_type)), (__temp369 == NULL ? NULL : g_object_ref (__temp369)));
	ValaStruct* __temp370 = NULL;
	ValaStruct* right = (__temp370 = VALA_STRUCT (vala_type_reference_get_data_type (right_type)), (__temp370 == NULL ? NULL : g_object_ref (__temp370)));
	if ((!vala_struct_is_floating_type (left) && !vala_struct_is_integer_type (left)) || (!vala_struct_is_floating_type (right) && !vala_struct_is_integer_type (right))) {
		gpointer __temp371;
		return VALA_TYPE_REFERENCE ((__temp371 = NULL, (left == NULL ? NULL : (left = (g_object_unref (left), NULL))), (right == NULL ? NULL : (right = (g_object_unref (right), NULL))), __temp371));
	}
	/* at least one operand not numeric*/
	if (vala_struct_is_floating_type (left) == vala_struct_is_floating_type (right)) {
		/* both operands integer or floating type*/
		if (vala_struct_get_rank (left) >= vala_struct_get_rank (right)) {
			ValaTypeReference* __temp372 = NULL;
			return (__temp372 = g_object_ref (left_type), (left == NULL ? NULL : (left = (g_object_unref (left), NULL))), (right == NULL ? NULL : (right = (g_object_unref (right), NULL))), __temp372);
		} else {
			ValaTypeReference* __temp373 = NULL;
			return (__temp373 = g_object_ref (right_type), (left == NULL ? NULL : (left = (g_object_unref (left), NULL))), (right == NULL ? NULL : (right = (g_object_unref (right), NULL))), __temp373);
		}
	} else {
		/* one integer and one floating type operand*/
		if (vala_struct_is_floating_type (left)) {
			ValaTypeReference* __temp374 = NULL;
			return (__temp374 = g_object_ref (left_type), (left == NULL ? NULL : (left = (g_object_unref (left), NULL))), (right == NULL ? NULL : (right = (g_object_unref (right), NULL))), __temp374);
		} else {
			ValaTypeReference* __temp375 = NULL;
			return (__temp375 = g_object_ref (right_type), (left == NULL ? NULL : (left = (g_object_unref (left), NULL))), (right == NULL ? NULL : (right = (g_object_unref (right), NULL))), __temp375);
		}
	}
	(left == NULL ? NULL : (left = (g_object_unref (left), NULL)));
	(right == NULL ? NULL : (right = (g_object_unref (right), NULL)));
}


static void vala_semantic_analyzer_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BINARY_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_binary_expression_get_left (expr))) || vala_code_node_get_error (VALA_CODE_NODE (vala_binary_expression_get_right (expr)))) {
		/* if there were any errors in inner expressions, skip type check */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		return;
	}
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) == vala_type_reference_get_data_type (self->priv->string_type) && vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS) {
		if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) != vala_type_reference_get_data_type (self->priv->string_type)) {
			(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Operands must be strings");
			return;
		}
		/* string concatenation: convert to a.concat (b) */
		ValaInvocationExpression* concat_call = vala_invocation_expression_new (VALA_EXPRESSION (vala_member_access_new (vala_binary_expression_get_left (expr), "concat", NULL)), NULL);
		vala_invocation_expression_add_argument (concat_call, vala_binary_expression_get_right (expr));
		vala_code_node_replace (vala_code_node_get_parent_node (VALA_CODE_NODE (expr)), VALA_CODE_NODE (expr), VALA_CODE_NODE (concat_call));
		vala_code_node_accept (VALA_CODE_NODE (concat_call), VALA_CODE_VISITOR (self));
		(concat_call == NULL ? NULL : (concat_call = (g_object_unref (concat_call), NULL)));
	} else {
		if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MINUS || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MUL || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_DIV) {
			ValaTypeReference* __temp376 = NULL;
			(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp376 = vala_semantic_analyzer_get_arithmetic_result_type (self, vala_expression_get_static_type (vala_binary_expression_get_left (expr)), vala_expression_get_static_type (vala_binary_expression_get_right (expr))))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
			(__temp376 == NULL ? NULL : (__temp376 = (g_object_unref (__temp376), NULL)));
			if (vala_expression_get_static_type (VALA_EXPRESSION (expr)) == NULL) {
				(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
				char* __temp379 = NULL;
				char* __temp378 = NULL;
				char* __temp377 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp379 = g_strdup_printf ("Arithmetic operation not supported for types `%s' and `%s'", (__temp377 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))), (__temp378 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))))));
				(__temp379 == NULL ? NULL : (__temp379 = (g_free (__temp379), NULL)));
				(__temp378 == NULL ? NULL : (__temp378 = (g_free (__temp378), NULL)));
				(__temp377 == NULL ? NULL : (__temp377 = (g_free (__temp377), NULL)));
				return;
			}
		} else {
			if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MOD || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_LEFT || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_RIGHT || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_XOR) {
				ValaTypeReference* __temp380 = NULL;
				(vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp380 = vala_semantic_analyzer_get_arithmetic_result_type (self, vala_expression_get_static_type (vala_binary_expression_get_left (expr)), vala_expression_get_static_type (vala_binary_expression_get_right (expr))))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
				(__temp380 == NULL ? NULL : (__temp380 = (g_object_unref (__temp380), NULL)));
				if (vala_expression_get_static_type (VALA_EXPRESSION (expr)) == NULL) {
					(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
					char* __temp383 = NULL;
					char* __temp382 = NULL;
					char* __temp381 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp383 = g_strdup_printf ("Arithmetic operation not supported for types `%s' and `%s'", (__temp381 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))), (__temp382 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))))));
					(__temp383 == NULL ? NULL : (__temp383 = (g_free (__temp383), NULL)));
					(__temp382 == NULL ? NULL : (__temp382 = (g_free (__temp382), NULL)));
					(__temp381 == NULL ? NULL : (__temp381 = (g_free (__temp381), NULL)));
					return;
				}
			} else {
				if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN_OR_EQUAL || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL) {
					if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) == vala_type_reference_get_data_type (self->priv->string_type) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) == vala_type_reference_get_data_type (self->priv->string_type)) {
						/* string comparison: convert to a.collate (b) OP 0 */
						ValaInvocationExpression* cmp_call = vala_invocation_expression_new (VALA_EXPRESSION (vala_member_access_new (vala_binary_expression_get_left (expr), "collate", NULL)), NULL);
						vala_invocation_expression_add_argument (cmp_call, vala_binary_expression_get_right (expr));
						(vala_binary_expression_set_left (expr, VALA_EXPRESSION (cmp_call)), vala_binary_expression_get_left (expr));
						ValaLiteralExpression* __temp384 = NULL;
						(vala_binary_expression_set_right (expr, VALA_EXPRESSION ((__temp384 = vala_literal_expression_new (VALA_LITERAL (vala_integer_literal_new ("0", NULL)), NULL)))), vala_binary_expression_get_right (expr));
						(__temp384 == NULL ? NULL : (__temp384 = (g_object_unref (__temp384), NULL)));
						vala_code_node_accept (VALA_CODE_NODE (vala_binary_expression_get_left (expr)), VALA_CODE_VISITOR (self));
						(cmp_call == NULL ? NULL : (cmp_call = (g_object_unref (cmp_call), NULL)));
					} else {
						/* TODO: check for integer or floating point type in expr.left */
						if (!vala_semantic_analyzer_check_binary_type (self, expr, "Relational operation")) {
							(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
							return;
						}
					}
					(vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
				} else {
					if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_EQUALITY || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_INEQUALITY) {
						/* relational operation */
						if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_right (expr)), vala_expression_get_static_type (vala_binary_expression_get_left (expr))) && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_left (expr)), vala_expression_get_static_type (vala_binary_expression_get_right (expr)))) {
							char* __temp387 = NULL;
							char* __temp386 = NULL;
							char* __temp385 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp387 = g_strdup_printf ("Equality operation: `%s' and `%s' are incompatible, comparison would always evaluate to false", (__temp385 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))), (__temp386 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))))));
							(__temp387 == NULL ? NULL : (__temp387 = (g_free (__temp387), NULL)));
							(__temp386 == NULL ? NULL : (__temp386 = (g_free (__temp386), NULL)));
							(__temp385 == NULL ? NULL : (__temp385 = (g_free (__temp385), NULL)));
							(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
							return;
						}
						if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) == vala_type_reference_get_data_type (self->priv->string_type) && vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) == vala_type_reference_get_data_type (self->priv->string_type)) {
							/* string comparison: convert to a.collate (b) OP 0 */
							ValaInvocationExpression* cmp_call = vala_invocation_expression_new (VALA_EXPRESSION (vala_member_access_new (vala_binary_expression_get_left (expr), "collate", NULL)), NULL);
							vala_invocation_expression_add_argument (cmp_call, vala_binary_expression_get_right (expr));
							(vala_binary_expression_set_left (expr, VALA_EXPRESSION (cmp_call)), vala_binary_expression_get_left (expr));
							ValaLiteralExpression* __temp388 = NULL;
							(vala_binary_expression_set_right (expr, VALA_EXPRESSION ((__temp388 = vala_literal_expression_new (VALA_LITERAL (vala_integer_literal_new ("0", NULL)), NULL)))), vala_binary_expression_get_right (expr));
							(__temp388 == NULL ? NULL : (__temp388 = (g_object_unref (__temp388), NULL)));
							vala_code_node_accept (VALA_CODE_NODE (vala_binary_expression_get_left (expr)), VALA_CODE_VISITOR (self));
							(cmp_call == NULL ? NULL : (cmp_call = (g_object_unref (cmp_call), NULL)));
						}
						(vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
					} else {
						if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_AND || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_OR) {
							/* integer type or flags type*/
							(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_binary_expression_get_left (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
						} else {
							if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_AND || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_OR) {
								if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_left (expr))) != vala_type_reference_get_data_type (self->priv->bool_type) || vala_type_reference_get_data_type (vala_expression_get_static_type (vala_binary_expression_get_right (expr))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
									(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
									vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Operands must be boolean");
								}
								(vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
							} else {
								g_assert_not_reached ();
							}
						}
					}
				}
			}
		}
	}
}


static void vala_semantic_analyzer_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_TYPE_CHECK (expr));
	if (vala_type_reference_get_data_type (vala_type_check_get_type_reference (expr)) == NULL) {
		/* if type resolving didn't succeed, skip this check */
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		return;
	}
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_reference_get_data_type (vala_type_check_get_type_reference (expr)))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static void vala_semantic_analyzer_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONDITIONAL_EXPRESSION (expr));
	if (vala_type_reference_get_data_type (vala_expression_get_static_type (vala_conditional_expression_get_condition (expr))) != vala_type_reference_get_data_type (self->priv->bool_type)) {
		(vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE), vala_code_node_get_error (VALA_CODE_NODE (expr)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_conditional_expression_get_condition (expr))), "Condition must be boolean");
		return;
	}
	/* FIXME: use greatest lower bound in the type hierarchy 
	 FIXME: support memory management */
	(vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_conditional_expression_get_true_expression (expr))), vala_expression_get_static_type (VALA_EXPRESSION (expr)));
}


static char* vala_semantic_analyzer_get_lambda_name (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	char* result = g_strdup_printf ("__lambda%d", self->priv->next_lambda_id);
	self->priv->next_lambda_id++;
	return result;
	(result == NULL ? NULL : (result = (g_free (result), NULL)));
}


static ValaMethod* vala_semantic_analyzer_find_current_method (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	ValaSymbol* __temp390 = NULL;
	ValaSymbol* sym = (__temp390 = self->priv->current_symbol, (__temp390 == NULL ? NULL : g_object_ref (__temp390)));
	while (sym != NULL) {
		if (VALA_IS_METHOD (vala_symbol_get_node (sym))) {
			ValaMethod* __temp391 = NULL;
			ValaMethod* __temp392 = NULL;
			return (__temp392 = (__temp391 = VALA_METHOD (vala_symbol_get_node (sym)), (__temp391 == NULL ? NULL : g_object_ref (__temp391))), (sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL))), __temp392);
		}
		ValaSymbol* __temp394 = NULL;
		ValaSymbol* __temp393 = NULL;
		sym = (__temp394 = (__temp393 = vala_symbol_get_parent_symbol (sym), (__temp393 == NULL ? NULL : g_object_ref (__temp393))), (sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL))), __temp394);
	}
	gpointer __temp395;
	return VALA_METHOD ((__temp395 = NULL, (sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL))), __temp395));
	(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
}


static gboolean vala_semantic_analyzer_is_in_constructor (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	ValaSymbol* __temp396 = NULL;
	ValaSymbol* sym = (__temp396 = self->priv->current_symbol, (__temp396 == NULL ? NULL : g_object_ref (__temp396)));
	while (sym != NULL) {
		if (VALA_IS_CONSTRUCTOR (vala_symbol_get_node (sym))) {
			gboolean __temp397;
			return (__temp397 = TRUE, (sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL))), __temp397);
		}
		ValaSymbol* __temp399 = NULL;
		ValaSymbol* __temp398 = NULL;
		sym = (__temp399 = (__temp398 = vala_symbol_get_parent_symbol (sym), (__temp398 == NULL ? NULL : g_object_ref (__temp398))), (sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL))), __temp399);
	}
	gboolean __temp400;
	return (__temp400 = FALSE, (sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL))), __temp400);
	(sym == NULL ? NULL : (sym = (g_object_unref (sym), NULL)));
}


static void vala_semantic_analyzer_real_visit_begin_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_LAMBDA_EXPRESSION (l));
	if (vala_expression_get_expected_type (VALA_EXPRESSION (l)) == NULL || !(VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (l)))))) {
		(vala_code_node_set_error (VALA_CODE_NODE (l), TRUE), vala_code_node_get_error (VALA_CODE_NODE (l)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (l)), "lambda expression not allowed in this context");
		return;
	}
	gboolean in_instance_method = FALSE;
	ValaMethod* current_method = vala_semantic_analyzer_find_current_method (self);
	if (current_method != NULL) {
		in_instance_method = vala_method_get_instance (current_method);
	} else {
		in_instance_method = vala_semantic_analyzer_is_in_constructor (self);
	}
	ValaCallback* __temp401 = NULL;
	ValaCallback* cb = (__temp401 = VALA_CALLBACK (vala_type_reference_get_data_type (vala_expression_get_expected_type (VALA_EXPRESSION (l)))), (__temp401 == NULL ? NULL : g_object_ref (__temp401)));
	ValaMethod* __temp402 = NULL;
	(vala_lambda_expression_set_method (l, (__temp402 = vala_method_new (vala_semantic_analyzer_get_lambda_name (self), vala_callback_get_return_type (cb), NULL))), vala_lambda_expression_get_method (l));
	(__temp402 == NULL ? NULL : (__temp402 = (g_object_unref (__temp402), NULL)));
	(vala_method_set_instance (vala_lambda_expression_get_method (l), vala_callback_get_instance (cb) && in_instance_method), vala_method_get_instance (vala_lambda_expression_get_method (l)));
	ValaSymbol* __temp403 = NULL;
	(vala_code_node_set_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)), (__temp403 = vala_symbol_new (VALA_CODE_NODE (vala_lambda_expression_get_method (l))))), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))));
	(__temp403 == NULL ? NULL : (__temp403 = (g_object_unref (__temp403), NULL)));
	(vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))), self->priv->current_symbol), vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)))));
	GList* lambda_params = vala_lambda_expression_get_parameters (l);
	GList* lambda_param_it = lambda_params;
	{
		GList* __temp406 = NULL;
		__temp406 = vala_callback_get_parameters (cb);
		GList* cb_param_it;
		for (cb_param_it = __temp406; cb_param_it != NULL; cb_param_it = cb_param_it->next) {
			ValaFormalParameter* cb_param = cb_param_it->data;
			{
				if (lambda_param_it == NULL) {
					break;
				}
				/* lambda expressions are allowed to have less parameters */
				const char* __temp404 = NULL;
				char* lambda_param = (__temp404 = ((char*) lambda_param_it->data), (__temp404 == NULL ? NULL : g_strdup (__temp404)));
				ValaFormalParameter* param = vala_formal_parameter_new (lambda_param, vala_formal_parameter_get_type_reference (cb_param), NULL);
				ValaSymbol* __temp405 = NULL;
				(vala_code_node_set_symbol (VALA_CODE_NODE (param), (__temp405 = vala_symbol_new (VALA_CODE_NODE (param)))), vala_code_node_get_symbol (VALA_CODE_NODE (param)));
				(__temp405 == NULL ? NULL : (__temp405 = (g_object_unref (__temp405), NULL)));
				vala_symbol_add (vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))), vala_formal_parameter_get_name (param), vala_code_node_get_symbol (VALA_CODE_NODE (param)));
				vala_method_add_parameter (vala_lambda_expression_get_method (l), param);
				lambda_param_it = lambda_param_it->next;
				(lambda_param == NULL ? NULL : (lambda_param = (g_free (lambda_param), NULL)));
				(param == NULL ? NULL : (param = (g_object_unref (param), NULL)));
			}
		}
		(__temp406 == NULL ? NULL : (__temp406 = (g_list_free (__temp406), NULL)));
	}
	if (lambda_param_it != NULL) {
		/* lambda expressions may not expect more parameters */
		(vala_code_node_set_error (VALA_CODE_NODE (l), TRUE), vala_code_node_get_error (VALA_CODE_NODE (l)));
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (l)), "lambda expression: too many parameters");
		(current_method == NULL ? NULL : (current_method = (g_object_unref (current_method), NULL)));
		(cb == NULL ? NULL : (cb = (g_object_unref (cb), NULL)));
		(lambda_params == NULL ? NULL : (lambda_params = (g_list_free (lambda_params), NULL)));
		return;
	}
	if (vala_lambda_expression_get_expression_body (l) != NULL) {
		ValaBlock* block = vala_block_new (NULL);
		ValaSymbol* __temp407 = NULL;
		(vala_code_node_set_symbol (VALA_CODE_NODE (block), (__temp407 = vala_symbol_new (VALA_CODE_NODE (block)))), vala_code_node_get_symbol (VALA_CODE_NODE (block)));
		(__temp407 == NULL ? NULL : (__temp407 = (g_object_unref (__temp407), NULL)));
		(vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (block)), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)))), vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (block))));
		if (vala_type_reference_get_data_type (vala_method_get_return_type (vala_lambda_expression_get_method (l))) != NULL) {
			ValaReturnStatement* __temp408 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp408 = vala_return_statement_new (vala_lambda_expression_get_expression_body (l), NULL))));
			(__temp408 == NULL ? NULL : (__temp408 = (g_object_unref (__temp408), NULL)));
		} else {
			ValaExpressionStatement* __temp409 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp409 = vala_expression_statement_new (vala_lambda_expression_get_expression_body (l), NULL))));
			(__temp409 == NULL ? NULL : (__temp409 = (g_object_unref (__temp409), NULL)));
		}
		(vala_method_set_body (vala_lambda_expression_get_method (l), block), vala_method_get_body (vala_lambda_expression_get_method (l)));
		(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	} else {
		(vala_method_set_body (vala_lambda_expression_get_method (l), vala_lambda_expression_get_statement_body (l)), vala_method_get_body (vala_lambda_expression_get_method (l)));
		(vala_symbol_set_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_body (vala_lambda_expression_get_method (l)))), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)))), vala_symbol_get_parent_symbol (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_body (vala_lambda_expression_get_method (l))))));
	}
	/* lambda expressions should be usable like MemberAccess of a method */
	(vala_expression_set_symbol_reference (VALA_EXPRESSION (l), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)))), vala_expression_get_symbol_reference (VALA_EXPRESSION (l)));
	(current_method == NULL ? NULL : (current_method = (g_object_unref (current_method), NULL)));
	(cb == NULL ? NULL : (cb = (g_object_unref (cb), NULL)));
	(lambda_params == NULL ? NULL : (lambda_params = (g_list_free (lambda_params), NULL)));
}


static void vala_semantic_analyzer_real_visit_begin_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	if (VALA_IS_MEMBER_ACCESS (vala_assignment_get_left (a))) {
		ValaMemberAccess* __temp410 = NULL;
		ValaMemberAccess* ma = (__temp410 = VALA_MEMBER_ACCESS (vala_assignment_get_left (a)), (__temp410 == NULL ? NULL : g_object_ref (__temp410)));
		if (vala_code_node_get_error (VALA_CODE_NODE (ma)) || vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)) == NULL) {
			(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
			(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
			return;
		}
		/* if no symbol found, skip this check */
		if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
			ValaSignal* __temp411 = NULL;
			ValaSignal* sig = (__temp411 = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))), (__temp411 == NULL ? NULL : g_object_ref (__temp411)));
			ValaTypeReference* __temp412 = NULL;
			(vala_expression_set_expected_type (vala_assignment_get_right (a), (__temp412 = vala_type_reference_new ())), vala_expression_get_expected_type (vala_assignment_get_right (a)));
			(__temp412 == NULL ? NULL : (__temp412 = (g_object_unref (__temp412), NULL)));
			ValaCallback* __temp413 = NULL;
			(vala_type_reference_set_data_type (vala_expression_get_expected_type (vala_assignment_get_right (a)), VALA_DATA_TYPE ((__temp413 = vala_signal_get_callback (sig)))), vala_type_reference_get_data_type (vala_expression_get_expected_type (vala_assignment_get_right (a))));
			(__temp413 == NULL ? NULL : (__temp413 = (g_object_unref (__temp413), NULL)));
			(sig == NULL ? NULL : (sig = (g_object_unref (sig), NULL)));
		}
		(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
	} else {
		if (VALA_IS_ELEMENT_ACCESS (vala_assignment_get_left (a))) {
		} else {
			/* do nothing*/
			if (VALA_IS_POINTER_INDIRECTION (vala_assignment_get_left (a))) {
			} else {
				/* do nothing*/
				(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "unsupported lvalue in assignment");
			}
		}
	}
}


static void vala_semantic_analyzer_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	if (vala_code_node_get_error (VALA_CODE_NODE (a)) || vala_code_node_get_error (VALA_CODE_NODE (vala_assignment_get_left (a))) || vala_code_node_get_error (VALA_CODE_NODE (vala_assignment_get_right (a)))) {
		(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
		return;
	}
	if (vala_assignment_get_operator (a) != VALA_ASSIGNMENT_OPERATOR_SIMPLE && VALA_IS_MEMBER_ACCESS (vala_assignment_get_left (a))) {
		/* transform into simple assignment
		 FIXME: only do this if the backend doesn't support
		 the assignment natively*/
		ValaMemberAccess* __temp414 = NULL;
		ValaMemberAccess* ma = (__temp414 = VALA_MEMBER_ACCESS (vala_assignment_get_left (a)), (__temp414 == NULL ? NULL : g_object_ref (__temp414)));
		if (!(VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))))) {
			ValaMemberAccess* old_value = vala_member_access_new (vala_member_access_get_inner (ma), vala_member_access_get_member_name (ma), NULL);
			ValaBinaryExpression* bin = vala_binary_expression_new (VALA_BINARY_OPERATOR_PLUS, VALA_EXPRESSION (old_value), VALA_EXPRESSION (vala_parenthesized_expression_new (vala_assignment_get_right (a), vala_code_node_get_source_reference (VALA_CODE_NODE (vala_assignment_get_right (a))))), NULL);
			if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_OR) {
				(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_BITWISE_OR), vala_binary_expression_get_operator (bin));
			} else {
				if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_AND) {
					(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_BITWISE_AND), vala_binary_expression_get_operator (bin));
				} else {
					if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_BITWISE_XOR) {
						(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_BITWISE_XOR), vala_binary_expression_get_operator (bin));
					} else {
						if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_ADD) {
							(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_PLUS), vala_binary_expression_get_operator (bin));
						} else {
							if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SUB) {
								(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_MINUS), vala_binary_expression_get_operator (bin));
							} else {
								if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_MUL) {
									(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_MUL), vala_binary_expression_get_operator (bin));
								} else {
									if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_DIV) {
										(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_DIV), vala_binary_expression_get_operator (bin));
									} else {
										if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_PERCENT) {
											(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_MOD), vala_binary_expression_get_operator (bin));
										} else {
											if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_LEFT) {
												(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_SHIFT_LEFT), vala_binary_expression_get_operator (bin));
											} else {
												if (vala_assignment_get_operator (a) == VALA_ASSIGNMENT_OPERATOR_SHIFT_RIGHT) {
													(vala_binary_expression_set_operator (bin, VALA_BINARY_OPERATOR_SHIFT_RIGHT), vala_binary_expression_get_operator (bin));
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
			(vala_assignment_set_right (a, VALA_EXPRESSION (bin)), vala_assignment_get_right (a));
			vala_code_node_accept (VALA_CODE_NODE (vala_assignment_get_right (a)), VALA_CODE_VISITOR (self));
			(vala_assignment_set_operator (a, VALA_ASSIGNMENT_OPERATOR_SIMPLE), vala_assignment_get_operator (a));
			(old_value == NULL ? NULL : (old_value = (g_object_unref (old_value), NULL)));
			(bin == NULL ? NULL : (bin = (g_object_unref (bin), NULL)));
		}
		(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
	}
	if (VALA_IS_MEMBER_ACCESS (vala_assignment_get_left (a))) {
		ValaMemberAccess* __temp415 = NULL;
		ValaMemberAccess* ma = (__temp415 = VALA_MEMBER_ACCESS (vala_assignment_get_left (a)), (__temp415 == NULL ? NULL : g_object_ref (__temp415)));
		if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
			ValaSignal* __temp416 = NULL;
			ValaSignal* sig = (__temp416 = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))), (__temp416 == NULL ? NULL : g_object_ref (__temp416)));
			if (vala_expression_get_symbol_reference (vala_assignment_get_right (a)) == NULL) {
				(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_assignment_get_right (a))), "unsupported expression for signal handler");
				(sig == NULL ? NULL : (sig = (g_object_unref (sig), NULL)));
				(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
				return;
			}
			ValaMethod* __temp417 = NULL;
			ValaMethod* m = (__temp417 = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_right (a)))), (__temp417 == NULL ? NULL : g_object_ref (__temp417)));
			if (vala_method_get_instance (m) && m->access != VALA_MEMBER_ACCESSIBILITY_PRIVATE) {
				/* TODO: generate wrapper function */
				(vala_code_node_set_error (VALA_CODE_NODE (ma), TRUE), vala_code_node_get_error (VALA_CODE_NODE (ma)));
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_assignment_get_right (a))), "public instance methods not yet supported as signal handlers");
				(sig == NULL ? NULL : (sig = (g_object_unref (sig), NULL)));
				(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
				(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
				return;
			}
			if (vala_method_get_instance (m)) {
				/* instance signal handlers must have the self
				 * parameter at the end
				 * do not use G_CONNECT_SWAPPED as this would
				 * rearrange the parameters for instance
				 * methods and non-instance methods
				 */
				(vala_method_set_instance_last (m, TRUE), vala_method_get_instance_last (m));
			}
			(sig == NULL ? NULL : (sig = (g_object_unref (sig), NULL)));
			(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
		} else {
			if (VALA_IS_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
				ValaProperty* __temp418 = NULL;
				ValaProperty* prop = (__temp418 = VALA_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))), (__temp418 == NULL ? NULL : g_object_ref (__temp418)));
				if (vala_property_get_set_accessor (prop) == NULL) {
					(vala_code_node_set_error (VALA_CODE_NODE (ma), TRUE), vala_code_node_get_error (VALA_CODE_NODE (ma)));
					char* __temp420 = NULL;
					char* __temp419 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (ma)), (__temp420 = g_strdup_printf ("Property `%s' is read-only", (__temp419 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))))));
					(__temp420 == NULL ? NULL : (__temp420 = (g_free (__temp420), NULL)));
					(__temp419 == NULL ? NULL : (__temp419 = (g_free (__temp419), NULL)));
					(prop == NULL ? NULL : (prop = (g_object_unref (prop), NULL)));
					(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
					return;
				}
				(prop == NULL ? NULL : (prop = (g_object_unref (prop), NULL)));
			} else {
				if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))) && vala_expression_get_static_type (vala_assignment_get_right (a)) == NULL) {
					ValaVariableDeclarator* __temp421 = NULL;
					ValaVariableDeclarator* decl = (__temp421 = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))), (__temp421 == NULL ? NULL : g_object_ref (__temp421)));
					ValaMemberAccess* __temp422 = NULL;
					ValaMemberAccess* right_ma = (__temp422 = VALA_MEMBER_ACCESS (vala_assignment_get_right (a)), (__temp422 == NULL ? NULL : g_object_ref (__temp422)));
					if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma)))) && VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl)))) {
						ValaMethod* __temp423 = NULL;
						ValaMethod* m = (__temp423 = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma)))), (__temp423 == NULL ? NULL : g_object_ref (__temp423)));
						ValaCallback* __temp424 = NULL;
						ValaCallback* cb = (__temp424 = VALA_CALLBACK (vala_type_reference_get_data_type (vala_variable_declarator_get_type_reference (decl))), (__temp424 == NULL ? NULL : g_object_ref (__temp424)));
						/* check whether method matches callback type */
						if (!vala_callback_matches_method (cb, m)) {
							(vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE), vala_code_node_get_error (VALA_CODE_NODE (decl)));
							char* __temp427 = NULL;
							char* __temp426 = NULL;
							char* __temp425 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp427 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp425 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp426 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
							(__temp427 == NULL ? NULL : (__temp427 = (g_free (__temp427), NULL)));
							(__temp426 == NULL ? NULL : (__temp426 = (g_free (__temp426), NULL)));
							(__temp425 == NULL ? NULL : (__temp425 = (g_free (__temp425), NULL)));
							(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
							(cb == NULL ? NULL : (cb = (g_object_unref (cb), NULL)));
							(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
							(right_ma == NULL ? NULL : (right_ma = (g_object_unref (right_ma), NULL)));
							(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
							return;
						}
						(vala_expression_set_static_type (vala_assignment_get_right (a), vala_variable_declarator_get_type_reference (decl)), vala_expression_get_static_type (vala_assignment_get_right (a)));
						(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
						(cb == NULL ? NULL : (cb = (g_object_unref (cb), NULL)));
					} else {
						(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Assignment: Invalid callback assignment attempt");
						(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
						(right_ma == NULL ? NULL : (right_ma = (g_object_unref (right_ma), NULL)));
						(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
						return;
					}
					(decl == NULL ? NULL : (decl = (g_object_unref (decl), NULL)));
					(right_ma == NULL ? NULL : (right_ma = (g_object_unref (right_ma), NULL)));
				} else {
					if (VALA_IS_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))) && vala_expression_get_static_type (vala_assignment_get_right (a)) == NULL) {
						ValaField* __temp428 = NULL;
						ValaField* f = (__temp428 = VALA_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))), (__temp428 == NULL ? NULL : g_object_ref (__temp428)));
						ValaMemberAccess* __temp429 = NULL;
						ValaMemberAccess* right_ma = (__temp429 = VALA_MEMBER_ACCESS (vala_assignment_get_right (a)), (__temp429 == NULL ? NULL : g_object_ref (__temp429)));
						if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma)))) && VALA_IS_CALLBACK (vala_type_reference_get_data_type (vala_field_get_type_reference (f)))) {
							ValaMethod* __temp430 = NULL;
							ValaMethod* m = (__temp430 = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma)))), (__temp430 == NULL ? NULL : g_object_ref (__temp430)));
							ValaCallback* __temp431 = NULL;
							ValaCallback* cb = (__temp431 = VALA_CALLBACK (vala_type_reference_get_data_type (vala_field_get_type_reference (f))), (__temp431 == NULL ? NULL : g_object_ref (__temp431)));
							/* check whether method matches callback type */
							if (!vala_callback_matches_method (cb, m)) {
								(vala_code_node_set_error (VALA_CODE_NODE (f), TRUE), vala_code_node_get_error (VALA_CODE_NODE (f)));
								char* __temp434 = NULL;
								char* __temp433 = NULL;
								char* __temp432 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp434 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp432 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp433 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
								(__temp434 == NULL ? NULL : (__temp434 = (g_free (__temp434), NULL)));
								(__temp433 == NULL ? NULL : (__temp433 = (g_free (__temp433), NULL)));
								(__temp432 == NULL ? NULL : (__temp432 = (g_free (__temp432), NULL)));
								(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
								(cb == NULL ? NULL : (cb = (g_object_unref (cb), NULL)));
								(f == NULL ? NULL : (f = (g_object_unref (f), NULL)));
								(right_ma == NULL ? NULL : (right_ma = (g_object_unref (right_ma), NULL)));
								(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
								return;
							}
							(vala_expression_set_static_type (vala_assignment_get_right (a), vala_field_get_type_reference (f)), vala_expression_get_static_type (vala_assignment_get_right (a)));
							(m == NULL ? NULL : (m = (g_object_unref (m), NULL)));
							(cb == NULL ? NULL : (cb = (g_object_unref (cb), NULL)));
						} else {
							(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Assignment: Invalid callback assignment attempt");
							(f == NULL ? NULL : (f = (g_object_unref (f), NULL)));
							(right_ma == NULL ? NULL : (right_ma = (g_object_unref (right_ma), NULL)));
							(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
							return;
						}
						(f == NULL ? NULL : (f = (g_object_unref (f), NULL)));
						(right_ma == NULL ? NULL : (right_ma = (g_object_unref (right_ma), NULL)));
					} else {
						if (vala_expression_get_static_type (vala_assignment_get_left (a)) != NULL && vala_expression_get_static_type (vala_assignment_get_right (a)) != NULL) {
							if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_assignment_get_right (a)), vala_expression_get_static_type (vala_assignment_get_left (a)))) {
								/* if there was an error on either side,
								 * i.e. a.{left|right}.static_type == null, skip type check */
								(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
								char* __temp437 = NULL;
								char* __temp436 = NULL;
								char* __temp435 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp437 = g_strdup_printf ("Assignment: Cannot convert from `%s' to `%s'", (__temp435 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_right (a)))), (__temp436 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_left (a)))))));
								(__temp437 == NULL ? NULL : (__temp437 = (g_free (__temp437), NULL)));
								(__temp436 == NULL ? NULL : (__temp436 = (g_free (__temp436), NULL)));
								(__temp435 == NULL ? NULL : (__temp435 = (g_free (__temp435), NULL)));
								(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
								return;
							}
							if (vala_semantic_analyzer_get_memory_management (self)) {
								if (vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_assignment_get_right (a)))) {
									/* rhs transfers ownership of the expression */
									if (!vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
										/* lhs doesn't own the value */
										(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
										vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Invalid assignment from owned expression to unowned variable");
									}
								} else {
									if (vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
									}
								}
							}
						}
					}
				}
			}
		}
		(ma == NULL ? NULL : (ma = (g_object_unref (ma), NULL)));
	} else {
		/* lhs wants to own the value
		 * rhs doesn't transfer the ownership
		 * code generator needs to add reference
		 * increment calls */
		if (VALA_IS_ELEMENT_ACCESS (vala_assignment_get_left (a))) {
			ValaElementAccess* __temp438 = NULL;
			ValaElementAccess* ea = (__temp438 = VALA_ELEMENT_ACCESS (vala_assignment_get_left (a)), (__temp438 == NULL ? NULL : g_object_ref (__temp438)));
			if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_assignment_get_right (a)), vala_expression_get_static_type (vala_assignment_get_left (a)))) {
				/* if there was an error on either side,
				 * i.e. a.{left|right}.static_type == null, skip type check */
				(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
				char* __temp441 = NULL;
				char* __temp440 = NULL;
				char* __temp439 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp441 = g_strdup_printf ("Assignment: Cannot convert from `%s' to `%s'", (__temp439 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_right (a)))), (__temp440 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_left (a)))))));
				(__temp441 == NULL ? NULL : (__temp441 = (g_free (__temp441), NULL)));
				(__temp440 == NULL ? NULL : (__temp440 = (g_free (__temp440), NULL)));
				(__temp439 == NULL ? NULL : (__temp439 = (g_free (__temp439), NULL)));
				(ea == NULL ? NULL : (ea = (g_object_unref (ea), NULL)));
				return;
			}
			if (vala_semantic_analyzer_get_memory_management (self)) {
				if (vala_type_reference_get_transfers_ownership (vala_expression_get_static_type (vala_assignment_get_right (a)))) {
					/* rhs transfers ownership of the expression */
					GList* args = vala_type_reference_get_type_arguments (vala_expression_get_static_type (vala_element_access_get_container (ea)));
					if (g_list_length (args) != 1) {
						(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (ea)), "internal error: array reference without type arguments");
						(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
						(ea == NULL ? NULL : (ea = (g_object_unref (ea), NULL)));
						return;
					}
					ValaTypeReference* __temp442 = NULL;
					ValaTypeReference* element_type = (__temp442 = VALA_TYPE_REFERENCE (args->data), (__temp442 == NULL ? NULL : g_object_ref (__temp442)));
					if (!vala_type_reference_get_takes_ownership (element_type)) {
						/* lhs doesn't own the value
						 * promote lhs type if it is a local variable
						 * error if it's not a local variable */
						if (!(VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_element_access_get_container (ea)))))) {
							(vala_code_node_set_error (VALA_CODE_NODE (a), TRUE), vala_code_node_get_error (VALA_CODE_NODE (a)));
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Invalid assignment from owned expression to unowned variable");
							(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
							(element_type == NULL ? NULL : (element_type = (g_object_unref (element_type), NULL)));
							(ea == NULL ? NULL : (ea = (g_object_unref (ea), NULL)));
							return;
						}
						(vala_type_reference_set_takes_ownership (element_type, TRUE), vala_type_reference_get_takes_ownership (element_type));
					}
					(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
					(element_type == NULL ? NULL : (element_type = (g_object_unref (element_type), NULL)));
				} else {
					if (vala_type_reference_get_takes_ownership (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
					}
				}
			}
			(ea == NULL ? NULL : (ea = (g_object_unref (ea), NULL)));
		} else {
			return;
		}
	}
	/* lhs wants to own the value
	 * rhs doesn't transfer the ownership
	 * code generator needs to add reference
	 * increment calls */
	(vala_expression_set_static_type (VALA_EXPRESSION (a), vala_expression_get_static_type (vala_assignment_get_left (a))), vala_expression_get_static_type (VALA_EXPRESSION (a)));
}


gboolean vala_semantic_analyzer_get_memory_management (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	return self->priv->_memory_management;
}


void vala_semantic_analyzer_set_memory_management (ValaSemanticAnalyzer* self, gboolean value)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	self->priv->_memory_management = value;
}


static void vala_semantic_analyzer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (object);
	switch (property_id) {
		case VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT:
		g_value_set_boolean (value, vala_semantic_analyzer_get_memory_management (self));
		break;
	}
}


static void vala_semantic_analyzer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (object);
	switch (property_id) {
		case VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT:
		vala_semantic_analyzer_set_memory_management (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_semantic_analyzer_class_init (ValaSemanticAnalyzerClass * klass)
{
	vala_semantic_analyzer_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (ValaSemanticAnalyzerPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_semantic_analyzer_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_semantic_analyzer_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_semantic_analyzer_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_source_file = vala_semantic_analyzer_real_visit_begin_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_source_file = vala_semantic_analyzer_real_visit_end_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_namespace = vala_semantic_analyzer_real_visit_begin_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_namespace = vala_semantic_analyzer_real_visit_end_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_class = vala_semantic_analyzer_real_visit_begin_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_class = vala_semantic_analyzer_real_visit_end_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_struct = vala_semantic_analyzer_real_visit_begin_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_struct = vala_semantic_analyzer_real_visit_end_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_interface = vala_semantic_analyzer_real_visit_begin_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_interface = vala_semantic_analyzer_real_visit_end_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_constant = vala_semantic_analyzer_real_visit_constant;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_semantic_analyzer_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_method = vala_semantic_analyzer_real_visit_begin_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_method = vala_semantic_analyzer_real_visit_end_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_creation_method = vala_semantic_analyzer_real_visit_begin_creation_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_creation_method = vala_semantic_analyzer_real_visit_end_creation_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_formal_parameter = vala_semantic_analyzer_real_visit_formal_parameter;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property = vala_semantic_analyzer_real_visit_end_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_property_accessor = vala_semantic_analyzer_real_visit_begin_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property_accessor = vala_semantic_analyzer_real_visit_end_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_constructor = vala_semantic_analyzer_real_visit_begin_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_constructor = vala_semantic_analyzer_real_visit_end_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_destructor = vala_semantic_analyzer_real_visit_begin_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_destructor = vala_semantic_analyzer_real_visit_end_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_named_argument = vala_semantic_analyzer_real_visit_named_argument;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_block = vala_semantic_analyzer_real_visit_begin_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_block = vala_semantic_analyzer_real_visit_end_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_variable_declarator = vala_semantic_analyzer_real_visit_variable_declarator;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_initializer_list = vala_semantic_analyzer_real_visit_begin_initializer_list;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_initializer_list = vala_semantic_analyzer_real_visit_end_initializer_list;
	VALA_CODE_VISITOR_CLASS (klass)->visit_expression_statement = vala_semantic_analyzer_real_visit_expression_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_if_statement = vala_semantic_analyzer_real_visit_if_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_while_statement = vala_semantic_analyzer_real_visit_while_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_for_statement = vala_semantic_analyzer_real_visit_for_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_foreach_statement = vala_semantic_analyzer_real_visit_begin_foreach_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_return_statement = vala_semantic_analyzer_real_visit_end_return_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_lock_statement = vala_semantic_analyzer_real_visit_lock_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_array_creation_expression = vala_semantic_analyzer_real_visit_begin_array_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_array_creation_expression = vala_semantic_analyzer_real_visit_end_array_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_boolean_literal = vala_semantic_analyzer_real_visit_boolean_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_character_literal = vala_semantic_analyzer_real_visit_character_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_integer_literal = vala_semantic_analyzer_real_visit_integer_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_real_literal = vala_semantic_analyzer_real_visit_real_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_string_literal = vala_semantic_analyzer_real_visit_string_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_null_literal = vala_semantic_analyzer_real_visit_null_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_literal_expression = vala_semantic_analyzer_real_visit_literal_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_parenthesized_expression = vala_semantic_analyzer_real_visit_parenthesized_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_member_access = vala_semantic_analyzer_real_visit_member_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_invocation_expression = vala_semantic_analyzer_real_visit_begin_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_invocation_expression = vala_semantic_analyzer_real_visit_end_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_element_access = vala_semantic_analyzer_real_visit_element_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_base_access = vala_semantic_analyzer_real_visit_base_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_postfix_expression = vala_semantic_analyzer_real_visit_postfix_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_object_creation_expression = vala_semantic_analyzer_real_visit_end_object_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_typeof_expression = vala_semantic_analyzer_real_visit_typeof_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_unary_expression = vala_semantic_analyzer_real_visit_unary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_cast_expression = vala_semantic_analyzer_real_visit_cast_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_pointer_indirection = vala_semantic_analyzer_real_visit_pointer_indirection;
	VALA_CODE_VISITOR_CLASS (klass)->visit_addressof_expression = vala_semantic_analyzer_real_visit_addressof_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_binary_expression = vala_semantic_analyzer_real_visit_binary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_type_check = vala_semantic_analyzer_real_visit_type_check;
	VALA_CODE_VISITOR_CLASS (klass)->visit_conditional_expression = vala_semantic_analyzer_real_visit_conditional_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_lambda_expression = vala_semantic_analyzer_real_visit_begin_lambda_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_assignment = vala_semantic_analyzer_real_visit_begin_assignment;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_assignment = vala_semantic_analyzer_real_visit_end_assignment;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT, g_param_spec_boolean ("memory-management", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_semantic_analyzer_init (ValaSemanticAnalyzer * self)
{
	self->priv = VALA_SEMANTIC_ANALYZER_GET_PRIVATE (self);
	self->priv->next_lambda_id = 0;
}


static void vala_semantic_analyzer_dispose (GObject * obj)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (obj);
	(self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL)));
	(self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL)));
	(self->priv->current_source_file == NULL ? NULL : (self->priv->current_source_file = (g_object_unref (self->priv->current_source_file), NULL)));
	(self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL)));
	(self->priv->current_class == NULL ? NULL : (self->priv->current_class = (g_object_unref (self->priv->current_class), NULL)));
	(self->priv->current_struct == NULL ? NULL : (self->priv->current_struct = (g_object_unref (self->priv->current_struct), NULL)));
	(self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL)));
	(self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL)));
	(self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL)));
	(self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL)));
	(self->priv->type_type == NULL ? NULL : (self->priv->type_type = (g_object_unref (self->priv->type_type), NULL)));
	(self->priv->pointer_type == NULL ? NULL : (self->priv->pointer_type = (g_object_unref (self->priv->pointer_type), NULL)));
	(self->priv->initially_unowned_type == NULL ? NULL : (self->priv->initially_unowned_type = (g_object_unref (self->priv->initially_unowned_type), NULL)));
	ValaSemanticAnalyzerClass * klass;
	GObjectClass * parent_class;
	klass = VALA_SEMANTIC_ANALYZER_CLASS (g_type_class_peek (VALA_TYPE_SEMANTIC_ANALYZER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_semantic_analyzer_get_type ()
{
	static GType vala_semantic_analyzer_type_id = 0;
	if (G_UNLIKELY (vala_semantic_analyzer_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSemanticAnalyzerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_semantic_analyzer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSemanticAnalyzer), 0, (GInstanceInitFunc) vala_semantic_analyzer_init };
		vala_semantic_analyzer_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaSemanticAnalyzer", &g_define_type_info, 0);
	}
	return vala_semantic_analyzer_type_id;
}




